﻿'*******************************************************************************
' *   This file is part of NRtfTree Library.
' *
' *   NRtfTree Library is free software; you can redistribute it and/or modify
' *   it under the terms of the GNU Lesser General Public License as published by
' *   the Free Software Foundation; either version 3 of the License, or
' *   (at your option) any later version.
' *
' *   NRtfTree Library is distributed in the hope that it will be useful,
' *   but WITHOUT ANY WARRANTY; without even the implied warranty of
' *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' *   GNU Lesser General Public License for more details.
' *
' *   You should have received a copy of the GNU Lesser General Public License
' *   along with this program. If not, see <http://www.gnu.org/licenses/>.
' *******************************************************************************

'*******************************************************************************
' * Library:		NRtfTree
' * Version:     v0.3.0
' * Date:		01/05/2009
' * Copyright:   2006-2009 Salvador Gomez
' * E-mail:      sgoliver.net@gmail.com
' * Home Page:	http://www.sgoliver.net
' * SF Project:	http://nrtftree.sourceforge.net
' *				http://sourceforge.net/projects/nrtftree
' * Class:		Demo
' * Description:	Programa principal de demostración de la librería.
'* Contributed by: Terence L. Wallace
' *******************************************************************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Net.Sgoliver.NRtfTree.Core
Imports Net.Sgoliver.NRtfTree.Util
Imports System.Drawing.Imaging
Imports System.IO

Namespace Net.Sgoliver.NRtfTree
    Namespace Demo

        Public Class Demo
#Region "Main Program"

            Shared Sub Main(ByVal args() As String)
                Dim [option] As String = ""

                Dim language As Integer = SelectLanguage()

                Do While Not [option].Equals("0")
                    If language = 1 Then
                        PrintEnglishMenu()
                    ElseIf language = 2 Then
                        PrintSpanishMenu()
                    End If

                    [option] = Console.In.ReadLine()

                    Console.WriteLine("")

                    Select Case [option]
                        Case "1"
                            ExtractDocumentProperties()
                        Case "2"
                            GenerateRtfTree()
                        Case "3"
                            ExtractPlainText()
                        Case "4"
                            ExtractDocumentOutline()
                        Case "5"
                            ExtractHyperlinks()
                        Case "6"
                            ExtractImages()
                        Case "7"
                            ExtractObjects()
                        Case "8"
                            TagFormat()
                        Case "9"
                            MergeDocuments()
                        Case "10"
                            ConvertToHtml()
                    End Select
                Loop
            End Sub

#End Region

#Region "Menu"

            Private Shared Function SelectLanguage() As Integer
                Dim language As Integer = 0

                PrintLanguageMenu()

                Dim [option] As String = Console.In.ReadLine()

                Console.WriteLine("")

                Select Case [option]
                    Case "1"
                        language = 1
                    Case "2"
                        language = 2
                End Select

                Return language
            End Function

            Private Shared Sub PrintLanguageMenu()
                Console.WriteLine("")
                Console.WriteLine("Select Language / Selecciona Idioma:")
                Console.WriteLine("")
                Console.WriteLine("   1. English.")
                Console.WriteLine("   2. Español.")
                Console.WriteLine("")
                Console.Write("> ")
            End Sub

            Private Shared Sub PrintEnglishMenu()
                Console.WriteLine("")
                Console.WriteLine("Select Demo:")
                Console.WriteLine("")
                Console.WriteLine("   1. Extract document properties.")
                Console.WriteLine("   2. Generate RTF tree.")
                Console.WriteLine("   3. Extract plain text.")
                Console.WriteLine("   4. Extract document outline.")
                Console.WriteLine("   5. Extract hyperlinks.")
                Console.WriteLine("   6. Extract images.")
                Console.WriteLine("   7. Extract embedded objects.")
                Console.WriteLine("   8. Tag format (RtfReader Example).")
                Console.WriteLine("   9. Merge documents.")
                Console.WriteLine("  10. Translate to HTML.")
                Console.WriteLine("")
                Console.WriteLine("   0. Quit.")
                Console.WriteLine("")
                Console.Write("Option> ")
            End Sub

            Private Shared Sub PrintSpanishMenu()
                Console.WriteLine("")
                Console.WriteLine("Selecciona ejemplo:")
                Console.WriteLine("")
                Console.WriteLine("   1. Extraer propiedades del documento.")
                Console.WriteLine("   2. Generar árbol RTF.")
                Console.WriteLine("   3. Extraer texto plano.")
                Console.WriteLine("   4. Extraer esquema del documento.")
                Console.WriteLine("   5. Extraer enlaces.")
                Console.WriteLine("   6. Extraer imágenes.")
                Console.WriteLine("   7. Extraer objetos incrustados.")
                Console.WriteLine("   8. Etiquetar formato (Ejemplo de RtfReader).")
                Console.WriteLine("   9. Combinar documentos.")
                Console.WriteLine("  10. Convertir a HTML.")
                Console.WriteLine("")
                Console.WriteLine("   0. Salir.")
                Console.WriteLine("")
                Console.Write("Opción> ")
            End Sub

#End Region

            Public Shared Sub ExtractDocumentProperties()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim info As InfoGroup = tree.GetInfoGroup()

                Console.WriteLine("Extracting document properties:")

                Console.WriteLine("Title: {0}", info.Title)
                Console.WriteLine("Author: {0}", info.Author)
                Console.WriteLine("Company: {0}", info.Company)
                Console.WriteLine("Comments: {0}", info.DocComment)
                Console.WriteLine("Created: {0}", info.CreationTime)
                Console.WriteLine("Revised: {0}", info.RevisionTime)

                Console.WriteLine("")
            End Sub

            Public Shared Sub GenerateRtfTree()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim sw As New StreamWriter("rtftree.txt")

                Console.WriteLine("Generating RTF tree...")

                sw.Write(tree.ToStringEx())
                sw.Flush()
                sw.Close()

                Console.WriteLine("File 'rtftree.txt' created.")

                Console.WriteLine("")
            End Sub

            Public Shared Sub ExtractPlainText()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim sw As New StreamWriter("rtftree.txt")

                Console.WriteLine("Extracting text...")

                sw.Write(tree.Text)
                sw.Flush()
                sw.Close()

                Console.WriteLine("File 'rtftext.txt' created.")

                Console.WriteLine("")
            End Sub

            Public Shared Sub ExtractDocumentOutline()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim sst As RtfStyleSheetTable = tree.GetStyleSheetTable()

                Dim heading1 As Integer = sst.IndexOf("heading 1")
                Dim heading2 As Integer = sst.IndexOf("heading 2")
                Dim heading3 As Integer = sst.IndexOf("heading 3")

                tree.MainGroup.RemoveChild(tree.MainGroup.SelectChildGroups("stylesheet")(0))

                Dim headingKeywords As RtfNodeCollection = tree.MainGroup.SelectNodes("s")

                For i As Integer = 0 To headingKeywords.Count - 1
                    Dim hk As RtfTreeNode = headingKeywords(i)

                    Dim text As New StringBuilder("")

                    If hk.Parameter = heading1 OrElse hk.Parameter = heading2 OrElse hk.Parameter = heading3 Then
                        Dim sibling As RtfTreeNode = hk.NextSibling

                        Do While sibling IsNot Nothing AndAlso Not sibling.NodeKey.Equals("pard")
                            If sibling.NodeType = RtfNodeType.Text Then
                                text.Append(sibling.NodeKey)
                            ElseIf sibling.NodeType = RtfNodeType.Group Then
                                text.Append(ExtractGroupText(sibling))
                            End If

                            sibling = sibling.NextSibling
                        Loop

                        If hk.Parameter = heading1 Then
                            Console.WriteLine("H1: {0}", text)
                        ElseIf hk.Parameter = heading2 Then
                            Console.WriteLine("    H2: {0}", text)
                        ElseIf hk.Parameter = heading3 Then
                            Console.WriteLine("        H3: {0}", text)
                        End If
                    End If
                Next i
            End Sub

            Private Shared Sub ExtractHyperlinks()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim fields As RtfNodeCollection = tree.MainGroup.SelectGroups("field")

                For Each node As RtfTreeNode In fields
                    'Extract URL

                    Dim fldInst As RtfTreeNode = node.ChildNodes(1)

                    Dim fldInstText As String = ExtractGroupText(fldInst)

                    Dim url As String = fldInstText.Split(New String() {" "}, StringSplitOptions.RemoveEmptyEntries)(1)

                    'Extract Link Text

                    Dim fldRslt As RtfTreeNode = node.SelectSingleChildGroup("fldrslt")

                    Dim linkText As String = ExtractGroupText(fldRslt)

                    Console.WriteLine("[" & linkText & ", " & url & "]")
                Next node
            End Sub

            Private Shared Function ExtractGroupText(ByVal group As RtfTreeNode) As String
                Dim sb As New StringBuilder("")

                For Each node As RtfTreeNode In group.ChildNodes
                    If node.NodeType = RtfNodeType.Text Then
                        sb.Append(node.NodeKey)
                    ElseIf node.NodeType = RtfNodeType.Group Then
                        sb.Append(ExtractGroupText(node))
                    End If
                Next node

                Return sb.ToString()
            End Function

            Private Shared Sub ExtractImages()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                Dim imageNodes As RtfNodeCollection = tree.RootNode.SelectNodes("pict")

                Console.WriteLine("Extracting images...")

                Dim i As Integer = 1
                For Each node As RtfTreeNode In imageNodes
                    Dim imageNode As New ImageNode(node.ParentNode)

                    If imageNode.ImageFormat Is ImageFormat.Png Then
                        imageNode.SaveImage("image" & i & ".png")

                        Console.WriteLine("File '" & "image" & i & ".png" & "' created.")

                        i += 1
                    End If
                Next node

                Console.WriteLine("")
            End Sub

            Private Shared Sub ExtractObjects()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc.rtf")

                'Busca el primer nodo de tipo objeto.
                Dim objects As RtfNodeCollection = tree.RootNode.SelectGroups("object")

                Console.WriteLine("Extracting objects...")

                Dim i As Integer = 1
                For Each node As RtfTreeNode In objects
                    'Se crea un nodo RTF especializado en imágenes
                    Dim objectNode As New ObjectNode(node)

                    Console.WriteLine("Found new object:")
                    Console.WriteLine("Object type: " & objectNode.ObjectType)
                    Console.WriteLine("Object class: " & objectNode.ObjectClass)

                    Dim data() As Byte = objectNode.GetByteData()

                    Dim binaryFile As New FileStream("object" & i & ".xls", FileMode.Create, FileAccess.ReadWrite)
                    Dim bw As New BinaryWriter(binaryFile)

                    For j As Integer = 38 To data.Length - 1
                        bw.Write(data(j))
                    Next j
                    bw.Flush()
                    bw.Close()

                    Console.WriteLine("File 'object" & i & ".xls' created.")

                    i += 1
                Next node

                Console.WriteLine("")
            End Sub

            Private Shared Sub TagFormat()
                Dim res As String = ""

                Dim parser As New MyParser(res)

                Dim reader As New RtfReader(parser)

                reader.LoadRtfFile(TemplateDirectory & "test-doc2.rtf")

                Console.WriteLine("Processing...")

                reader.Parse()

                Dim sw As New StreamWriter("taggedfile.txt")
                sw.Write(parser.doc)
                sw.Flush()
                sw.Close()

                Console.WriteLine("File 'taggedfile.txt' created.")

                Console.WriteLine("")
            End Sub

            Private Shared Sub MergeDocuments()
                Dim merger As New RtfMerger(TemplateDirectory & "test-doc3.rtf", "merge-result.rtf", True)

                merger.AddPlaceHolder("[TagTextRTF1]", TemplateDirectory & "merge1.rtf")
                merger.AddPlaceHolder("[TagTextRTF2]", TemplateDirectory & "merge2.rtf")

                Console.WriteLine("Processing...")

                merger.MergeRtfDoc()

                Console.WriteLine("File 'merge-result.txt' created.")

                Console.WriteLine("")
            End Sub

            Private Shared Sub ConvertToHtml()
                Dim tree As New RtfTree()
                tree.LoadRtfFile(TemplateDirectory & "test-doc2.rtf")

                Dim rtfToHtml As New Rtf2Html()

                Console.WriteLine("Processing...")
                rtfToHtml.IncrustImages = False
                Dim html As String = rtfToHtml.Convert(tree.Rtf)

                Dim sw As New StreamWriter("test.html", False)
                sw.Write(html)
                sw.Flush()
                sw.Close()

                Console.WriteLine("File 'test.html' created.")

                Console.WriteLine("")
            End Sub

        End Class
    End Namespace
End Namespace
