'*******************************************************************************
' *   This file is part of NRtfTree Library.
' *
' *   NRtfTree Library is free software; you can redistribute it and/or modify
' *   it under the terms of the GNU Lesser General Public License as published by
' *   the Free Software Foundation; either version 3 of the License, or
' *   (at your option) any later version.
' *
' *   NRtfTree Library is distributed in the hope that it will be useful,
' *   but WITHOUT ANY WARRANTY; without even the implied warranty of
' *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' *   GNU Lesser General Public License for more details.
' *
' *   You should have received a copy of the GNU Lesser General Public License
' *   along with this program. If not, see <http://www.gnu.org/licenses/>.
' *******************************************************************************

'*******************************************************************************
' * Library:		NRtfTree
' * Version:     v0.3.0
' * Date:		01/05/2009
' * Copyright:   2006-2009 Salvador Gomez
' * E-mail:      sgoliver.net@gmail.com
' * Home Page:	http://www.sgoliver.net
' * SF Project:	http://nrtftree.sourceforge.net
' *				http://sourceforge.net/projects/nrtftree
' * Class:		MyParser
' * Description:	Parser para la traduccin de RTF a Pseudo XML. (Demo NRtfTree)
' * Contributed by: Terence L. Wallace
' *******************************************************************************


Imports Microsoft.VisualBasic
Imports System
Imports Net.Sgoliver.NRtfTree.Core
Imports Net.Sgoliver.NRtfTree.Util

Namespace Net.Sgoliver.NRtfTree
	Namespace Demo
		Public Class MyParser
			Inherits SarParser
			Public doc As String = ""

			Private enTexto As Boolean = False

			Private negrita As Boolean = False
			Private cursiva As Boolean = False
			Private subrayado As Boolean = False

			Public Sub New(ByVal doc As String)
				Me.doc = doc
			End Sub

			Public Overrides Sub StartRtfDocument()
				doc &= "<doc>" & Constants.vbCrLf
			End Sub

			Public Overrides Sub EndRtfDocument()
				doc &= "</doc>"
			End Sub

			Public Overrides Sub StartRtfGroup()

			End Sub

			Public Overrides Sub EndRtfGroup()
				If enTexto Then
					If negrita Then
						doc &= "</b>"
					End If

					If cursiva Then
						doc &= "</i>"
					End If

					If subrayado Then
						doc &= "</u>"
					End If

					negrita = False
					cursiva = False
					subrayado = False
				End If
			End Sub

			Public Overrides Sub RtfControl(ByVal key As String, ByVal hasParam As Boolean, ByVal param As Integer)
				If key = "'" Then 'Caracter especial
					doc &= translateAnsiCode(param)
				End If
			End Sub

			Private Function translateAnsiCode(ByVal cod As Integer) As String
				Dim res As String = ""

				Select Case cod
					Case 193
						res = ""
					Case 201
						res = ""
					Case 205
						res = ""
					Case 211
						res = ""
					Case 218
						res = ""
					Case 225
						res = ""
					Case 233
						res = ""
					Case 237
						res = ""
					Case 243
						res = ""
					Case 250
						res = ""
					Case 241
						res = ""
					Case 209
						res = ""
				End Select

				Return res
			End Function

			Public Overrides Sub RtfKeyword(ByVal key As String, ByVal hasParam As Boolean, ByVal param As Integer)
				If key.Equals("pard") Then
					enTexto = True
				End If

				If enTexto Then
					Select Case key
						Case "b"
							If (Not hasParam) OrElse (hasParam AndAlso param = 1) Then
								doc &= "<b>"
								negrita = True
							Else
								doc &= "</b>"
								negrita = False
							End If
						Case "i"
							If (Not hasParam) OrElse (hasParam AndAlso param = 1) Then
								doc &= "<i>"
								cursiva = True
							Else
								doc &= "</i>"
								cursiva = False
							End If
						Case "ul"
							doc &= "<u>"
							subrayado = True
						Case "ulnone"
							doc &= "</u>"
							subrayado = False
						Case "par"
							doc &= Constants.vbCrLf
					End Select
				End If
			End Sub

			Public Overrides Sub RtfText(ByVal text As String)
				If enTexto Then
					doc &= text
				End If
			End Sub
		End Class
	End Namespace
End Namespace
