'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: InfoGroup
'* Description: Class to encapsulate all the information contained in a
'* RTF group type \ info ".
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Text

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' A class that encapsulates all the information contained in a group "\ info" of an RTF document.
        ''' </summary>
        Public Class InfoGroup
#Region "Attributes private"

            Private _title As String = ""
            Private _subject As String = ""
            Private _author As String = ""
            Private _manager As String = ""
            Private _company As String = ""
            Private _operator As String = ""
            Private _category As String = ""
            Private _keywords As String = ""
            Private _comment As String = ""
            Private _doccomm As String = ""
            Private _hlinkbase As String = ""
            Private _creatim As DateTime = DateTime.MinValue
            Private _revtim As DateTime = DateTime.MinValue
            Private _printim As DateTime = DateTime.MinValue
            Private _buptim As DateTime = DateTime.MinValue
            Private _version As Integer = -1
            Private _vern As Integer = -1
            Private _edmins As Integer = -1
            Private _nofpages As Integer = -1
            Private _nofwords As Integer = -1
            Private _nofchars As Integer = -1
            Private _id As Integer = -1

#End Region

#Region "Properties"

            ''' <summary>
            ''' Title of document.
            ''' </summary>
            Public Property Title() As String
                Get
                    Return _title
                End Get
                Set(ByVal value As String)
                    _title = value
                End Set
            End Property

            ''' <summary>
            ''' Theme of the document.
            ''' </summary>
            Public Property Subject() As String
                Get
                    Return _subject
                End Get
                Set(ByVal value As String)
                    _subject = value
                End Set
            End Property

            ''' <summary>
            ''' Author.
            ''' </summary>
            Public Property Author() As String
                Get
                    Return _author
                End Get
                Set(ByVal value As String)
                    _author = value
                End Set
            End Property

            ''' <summary>
            ''' Manager of the author.
            ''' </summary>
            Public Property Manager() As String
                Get
                    Return _manager
                End Get
                Set(ByVal value As String)
                    _manager = value
                End Set
            End Property

            ''' <summary>
            ''' Carrier of the author.
            ''' </summary>
            Public Property Company() As String
                Get
                    Return _company
                End Get
                Set(ByVal value As String)
                    _company = value
                End Set
            End Property

            ''' <summary>
            ''' ltima realizao person who changes to the document.
            ''' </summary>
            Public Property [Operator]() As String
                Get
                    Return _operator
                End Get
                Set(ByVal value As String)
                    _operator = value
                End Set
            End Property

            ''' <summary>
            ''' Categorical document.
            ''' </summary>
            Public Property Category() As String
                Get
                    Return _category
                End Get
                Set(ByVal value As String)
                    _category = value
                End Set
            End Property

            ''' <summary>
            ''' Keywords for the document.
            ''' </summary>
            Public Property Keywords() As String
                Get
                    Return _keywords
                End Get
                Set(ByVal value As String)
                    _keywords = value
                End Set
            End Property

            ''' <summary>
            ''' Comments.
            ''' </summary>
            Public Property Comment() As String
                Get
                    Return _comment
                End Get
                Set(ByVal value As String)
                    _comment = value
                End Set
            End Property

            ''' <summary>
            ''' Comments are listed in the text box "Summary Info" or "Properties" from Microsoft Word.
            ''' </summary>
            Public Property DocComment() As String
                Get
                    Return _doccomm
                End Get
                Set(ByVal value As String)
                    _doccomm = value
                End Set
            End Property

            ''' <summary>
            ''' The base address used on the routes on the links in the document. It may be a local path or URL.
            ''' </summary>
            Public Property HlinkBase() As String
                Get
                    Return _hlinkbase
                End Get
                Set(ByVal value As String)
                    _hlinkbase = value
                End Set
            End Property

            ''' <summary>
            ''' Date / time of creation of the document.
            ''' </summary>
            Public Property CreationTime() As DateTime
                Get
                    Return _creatim
                End Get
                Set(ByVal value As DateTime)
                    _creatim = value
                End Set
            End Property

            ''' <summary>
            ''' Date / Time revisin the document.
            ''' </summary>
            Public Property RevisionTime() As DateTime
                Get
                    Return _revtim
                End Get
                Set(ByVal value As DateTime)
                    _revtim = value
                End Set
            End Property

            ''' <summary>
            ''' Date / Time ltima impresin the document.
            ''' </summary>
            Public Property LastPrintTime() As DateTime
                Get
                    Return _printim
                End Get
                Set(ByVal value As DateTime)
                    _printim = value
                End Set
            End Property

            ''' <summary>
            ''' Date / Time ltima copy.
            ''' </summary>
            Public Property BackupTime() As DateTime
                Get
                    Return _buptim
                End Get
                Set(ByVal value As DateTime)
                    _buptim = value
                End Set
            End Property

            ''' <summary>
            ''' Version of the document.
            ''' </summary>
            Public Property Version() As Integer
                Get
                    Return _version
                End Get
                Set(ByVal value As Integer)
                    _version = value
                End Set
            End Property

            ''' <summary>
            ''' Internal version of the document.
            ''' </summary>
            Public Property InternalVersion() As Integer
                Get
                    Return _vern
                End Get
                Set(ByVal value As Integer)
                    _vern = value
                End Set
            End Property

            ''' <summary>
            ''' Total time Edición the document (in minutes).
            ''' </summary>
            Public Property EditingTime() As Integer
                Get
                    Return _edmins
                End Get
                Set(ByVal value As Integer)
                    _edmins = value
                End Set
            End Property

            ''' <summary>
            ''' Number of pages in the document.
            ''' </summary>
            Public Property NumberOfPages() As Integer
                Get
                    Return _nofpages
                End Get
                Set(ByVal value As Integer)
                    _nofpages = value
                End Set
            End Property

            ''' <summary>
            ''' Numbers of words in the document.
            ''' </summary>
            Public Property NumberOfWords() As Integer
                Get
                    Return _nofwords
                End Get
                Set(ByVal value As Integer)
                    _nofwords = value
                End Set
            End Property

            ''' <summary>
            ''' Number of characters in the document.
            ''' </summary>
            Public Property NumberOfChars() As Integer
                Get
                    Return _nofchars
                End Get
                Set(ByVal value As Integer)
                    _nofchars = value
                End Set
            End Property

            ''' <summary>
            ''' Internal identifi document.
            ''' </summary>
            Public Property Id() As Integer
                Get
                    Return _id
                End Get
                Set(ByVal value As Integer)
                    _id = value
                End Set
            End Property

#End Region

#Region "Public Methods"

            ''' <summary>
            ''' Representacin Returns the node as a string of characters.
            ''' </summary>
            ''' <returns> Representacin node as a string of characters. </returns>
            Public Overrides Function ToString() As String
                Dim str As New StringBuilder()

                str.AppendLine("Title     : " & Me.Title)
                str.AppendLine("Subject   : " & Me.Subject)
                str.AppendLine("Author    : " & Me.Author)
                str.AppendLine("Manager   : " & Me.Manager)
                str.AppendLine("Company   : " & Me.Company)
                str.AppendLine("Operator  : " & Me.Operator)
                str.AppendLine("Category  : " & Me.Category)
                str.AppendLine("Keywords  : " & Me.Keywords)
                str.AppendLine("Comment   : " & Me.Comment)
                str.AppendLine("DComment  : " & Me.DocComment)
                str.AppendLine("HLinkBase : " & Me.HlinkBase)
                str.AppendLine("Created   : " & Me.CreationTime.ToString())
                str.AppendLine("Revised   : " & Me.RevisionTime.ToString())
                str.AppendLine("Printed   : " & Me.LastPrintTime.ToString())
                str.AppendLine("Backup    : " & Me.BackupTime.ToString())
                str.AppendLine("Version   : " & Me.Version)
                str.AppendLine("IVersion  : " & Me.InternalVersion)
                str.AppendLine("Editing   : " & Me.EditingTime)
                str.AppendLine("Num Pages : " & Me.NumberOfPages)
                str.AppendLine("Num Words : " & Me.NumberOfWords)
                str.AppendLine("Num Chars : " & Me.NumberOfChars)
                str.AppendLine("Id        : " & Me.Id)

                Return str.ToString()
            End Function

#End Region
        End Class
    End Namespace
End Namespace
