'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: ObjectNode
'* Description: RTF node contains specialized information of an object.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Text
Imports Net.Sgoliver.NRtfTree.Core
Imports System.Globalization

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' Encapsulates a node Object type RTF (Keyword '\ object ")
        ''' </summary>
        Public Class ObjectNode
            Inherits Net.Sgoliver.NRtfTree.Core.RtfTreeNode
#Region "Private Attributes"

            ''' <summary>
            ''' Array of bytes with information of the object.
            ''' </summary>
            Private objdata() As Byte

#End Region

#Region "Constructors"

            ''' <summary>
            ''' ObjectNode class constructor.
            ''' </summary>
            ''' <param name="node"> Node obtendrn RTF from which the image data. </param>
            Public Sub New(ByVal node As RtfTreeNode)
                If node IsNot Nothing Then
                    'Assign all fields in the node
                    Me.NodeKey = node.NodeKey
                    Me.HasParameter = node.HasParameter
                    Me.Parameter = node.Parameter
                    Me.ParentNode = node.ParentNode
                    Me.RootNode = node.RootNode
                    Me.NodeType = node.NodeType

                    Me.ChildNodes = New RtfNodeCollection()
                    Me.ChildNodes.AddRange(node.ChildNodes)

                    'Get the object's data as an array of bytes
                    getObjectData()
                End If
            End Sub

#End Region

#Region "Properties"

            ''' <summary>
            ''' Returns the type of object.
            ''' </summary>
            Public ReadOnly Property ObjectType() As String
                Get
                    If Me.SelectSingleChildNode("objemb") IsNot Nothing Then
                        Return "objemb"
                    End If
                    If Me.SelectSingleChildNode("objlink") IsNot Nothing Then
                        Return "objlink"
                    End If
                    If Me.SelectSingleChildNode("objautlink") IsNot Nothing Then
                        Return "objautlink"
                    End If
                    If Me.SelectSingleChildNode("objsub") IsNot Nothing Then
                        Return "objsub"
                    End If
                    If Me.SelectSingleChildNode("objpub") IsNot Nothing Then
                        Return "objpub"
                    End If
                    If Me.SelectSingleChildNode("objicemb") IsNot Nothing Then
                        Return "objicemb"
                    End If
                    If Me.SelectSingleChildNode("objhtml") IsNot Nothing Then
                        Return "objhtml"
                    End If
                    If Me.SelectSingleChildNode("objocx") IsNot Nothing Then
                        Return "objocx"
                    Else
                        Return ""
                    End If
                End Get
            End Property

            ''' <summary>
            ''' Returns the object's class.
            ''' </summary>
            Public ReadOnly Property ObjectClass() As String
                Get
                    'Format: (\ * \ objclass Paint.Picture)

                    Dim node As RtfTreeNode = Me.SelectSingleNode("objclass")

                    If node IsNot Nothing Then
                        Return node.NextSibling.NodeKey
                    Else
                        Return ""
                    End If
                End Get
            End Property

            ''' <summary>
            ''' Returns the group node that encapsulates RTF "\ result" of the object.
            ''' </summary>
            Public ReadOnly Property ResultNode() As RtfTreeNode
                Get
                    Dim node As RtfTreeNode = Me.SelectSingleNode("result")

                    'If there is a node "\ result" RTF group recovered later.
                    If node IsNot Nothing Then
                        node = node.ParentNode
                    End If

                    Return node
                End Get
            End Property

            ''' <summary>
            ''' Returns a string with the contents of the object in hexadecimal.
            ''' </summary>
            Public ReadOnly Property HexData() As String
                Get
                    Dim Text As String = ""

                    'Find the node "\ objdata"
                    Dim objdataNode As RtfTreeNode = Me.SelectSingleNode("objdata")

                    'If there is a node
                    If objdataNode IsNot Nothing Then
                        'We look for the data in hexadecimal (son of the group ltimo \ objdata)
                        Text = objdataNode.ParentNode.LastChild.NodeKey
                    End If

                    Return Text
                End Get
            End Property

#End Region

#Region "method published"

            ''' <summary>
            ''' Returns an array of bytes with the contents of the object.
            ''' </summary>
            ''' <returns> Array of bytes with the contents of the object. </returns>
            Public Function GetByteData() As Byte()
                Return objdata
            End Function

#End Region

#Region "Private methods, described"

            ''' <summary>
            ''' Gets the binary data of the object from the information contained in the node RTF.
            ''' </summary>
            Private Sub getObjectData()
                'Format: (' ( '\ object ( <objtype> & <objmod> ? & <objclass> ? & <objname> ? & <objtime> ? & <objsize> ? & <rsltmod> ?) ( '(\ *' \ Objdata ( <objalias> ? & <objsect> ?) <data> ')') <result> ')')

                Dim Text As String = ""

                If Me.FirstChild.NodeKey = "object" Then
                    'Find the node "\ objdata"
                    Dim objdataNode As RtfTreeNode = Me.SelectSingleNode("objdata")

                    'If there is a node
                    If objdataNode IsNot Nothing Then
                        'We look for the data in hexadecimal (son of the group ltimo \ objdata)
                        Text = objdataNode.ParentNode.LastChild.NodeKey

                        Dim dataSize As Integer = Text.Length \ 2
                        objdata = New Byte(dataSize - 1) {}

                        Dim sbaux As New StringBuilder(2)

                        For i As Integer = 0 To Text.Length - 1
                            sbaux.Append(Text.Chars(i))

                            If sbaux.Length = 2 Then
                                objdata(i \ 2) = Byte.Parse(sbaux.ToString(), NumberStyles.HexNumber)
                                sbaux.Remove(0, 2)
                            End If
                        Next i
                    End If
                End If
            End Sub

#End Region
        End Class
    End Namespace
End Namespace
