'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfColorTable
'* Description: Color Chart of an RTF document.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Drawing

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' Color table a RTF document.
        ''' </summary>
        Public Class RtfColorTable
            ''' <summary>
            ''' Internal list of colors.
            ''' </summary>
            Private colors As List(Of Integer)

            ''' <summary>
            ''' RtfColorTable class constructor.
            ''' </summary>
            Public Sub New()
                colors = New List(Of Integer)()
            End Sub

            ''' <summary>
            ''' Inserts a new color in the color table.
            ''' </summary>
            ''' <param name="color"> New color insert. </param>
            Public Sub AddColor(ByVal color As Color)
                colors.Add(color.ToArgb())
            End Sub

            ''' <summary>
            ''' Gets the nth color in the color table.
            ''' </summary>
            ''' <param name="index"> Color index to retrieve. </param>
            ''' <returns> Nth color in the color table. </returns>
            Default Public ReadOnly Property Item(ByVal index As Integer) As Color
                Get
                    Return Color.FromArgb(colors(index))
                End Get
            End Property

            ''' <summary>
            ''' Numbers of colors in the table.
            ''' </summary>
            Public ReadOnly Property Count() As Integer
                Get
                    Return colors.Count
                End Get
            End Property

            ''' <summary>
            ''' Gets the index of a given color in the table.
            ''' </summary>
            ''' <param name="color"> Color to view. </param>
            ''' <returns> Color index consulted. </returns>
            Public Function IndexOf(ByVal color As Color) As Integer
                Return colors.IndexOf(color.ToArgb())
            End Function
        End Class
    End Namespace
End Namespace
