'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfFontTable
'* Description: Table of Sources from a RTF document.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Collections

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' Table of sources of an RTF document.
        ''' </summary>
        Public Class RtfFontTable
            ''' <summary>
            ''' Internal list of fonts.
            ''' </summary>
            Private fonts As Dictionary(Of Integer, String)

            ''' <summary>
            ''' RtfFontTable class constructor.
            ''' </summary>
            Public Sub New()
                fonts = New Dictionary(Of Integer, String)()
            End Sub

            ''' <summary>
            ''' Inserts a new font in the font table.
            ''' </summary>
            ''' <param name="name"> New source to add. </param>
            Public Sub AddFont(ByVal name As String)
                fonts.Add(newFontIndex(), name)
            End Sub

            ''' <summary>
            ''' Inserts a new font in the font table.
            ''' </summary>
            ''' <param name="index"> Index the source to add. </param>
            ''' <param name="name"> New source to add. </param>
            Public Sub AddFont(ByVal index As Integer, ByVal name As String)
                fonts.Add(index, name)
            End Sub

            ''' <summary>
            ''' Gets the nth source table sources.
            ''' </summary>
            ''' <param name="index"> Index to retrieve the source. </param>
            ''' <returns> Source nth table sources. </returns>
            Default Public ReadOnly Property Item(ByVal index As Integer) As String
                Get
                    Return fonts(index)
                End Get
            End Property

            ''' <summary>
            ''' Numbers of sources in the table.
            ''' </summary>
            Public ReadOnly Property Count() As Integer
                Get
                    Return fonts.Count
                End Get
            End Property

            ''' <summary>
            ''' Gets the index of a particular font in the table.
            ''' </summary>
            ''' <param name="name"> Source to consult. </param>
            ''' <returns> Index the source consulted. </returns>
            Public Function IndexOf(ByVal name As String) As Integer
                Dim intIndex As Integer = -1
                Dim fntIndex As IEnumerator = fonts.GetEnumerator()

                fntIndex.Reset()
                Do While fntIndex.MoveNext() = True
                    If (CType(fntIndex.Current, KeyValuePair(Of Integer, String))).Value.Equals(name) Then
                        intIndex = System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, String))).Key)
                        Exit Do
                    End If
                Loop

                Return intIndex
            End Function

            ''' <summary>
            ''' Gets a est index is not used in the table.
            ''' </summary>
            ''' <returns> Gets a est index is not used in the table. </returns>
            Private Function newFontIndex() As Integer
                Dim intIndex As Integer = -1
                Dim fntIndex As IEnumerator = fonts.GetEnumerator()

                fntIndex.Reset()
                Do While fntIndex.MoveNext() = True
                    If System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, String))).Key) > intIndex Then
                        intIndex = System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, String))).Key)
                    End If
                Loop

                Return (intIndex + 1)
            End Function
        End Class
    End Namespace
End Namespace
