'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfLex
'* Description: RTF document lxico analyzer.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.Text

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' Lxico analyzer (tokenizer) to RTF format documents. Parses the document and returns of
        ''' sequentially all read RTF elements (tokens).
        ''' </summary>
        Public Class RtfLex
#Region "Attributes private"

            ''' <summary>
            ''' Open file.
            ''' </summary>
            Private rtf As TextReader

            ''' <summary>
            ''' StringBuilder to build the keyword / snippet
            ''' </summary>
            Private keysb As StringBuilder

            ''' <summary>
            ''' StringBuilder to build your keyword parmetro
            ''' </summary>
            Private parsb As StringBuilder

            ''' <summary>
            ''' Character read from the document
            ''' </summary>
            Private c As Integer

#End Region

#Region "Constants"

            ''' <summary>
            ''' Mark end of file.
            ''' </summary>
            Private Const Eof As Integer = -1

#End Region

#Region "Constructors"

            ''' <summary>
            ''' RtfLex class constructor
            ''' </summary>
            ''' <param name="rtfReader"> Stream of file to analyze. </param>
            Public Sub New(ByVal rtfReader As TextReader)
                rtf = rtfReader

                keysb = New StringBuilder()
                parsb = New StringBuilder()

                'He reads the first character of the document
                c = rtf.Read()
            End Sub

#End Region

#Region "alternative methods of Públicos"

            ''' <summary>
            ''' Read a token of RTF document.
            ''' </summary>
            ''' <returns> Next token read from the document. </returns>
            Public Function NextToken() As RtfToken
                'It creates a new token to return
                Dim token As New RtfToken()

                'It ignores the carriage returns, tabs and null characters
                Do While c = Asc(ControlChars.Cr) OrElse c = Asc(ControlChars.Lf) OrElse c = Asc(ControlChars.Tab) OrElse c = Asc(ControlChars.NullChar)
                    c = rtf.Read()
                Loop

                'This is the character read
                If c <> Eof Then
                    Select Case c
                        Case AscW("{")
                            token.Type = RtfTokenType.GroupStart
                            c = rtf.Read()
                        Case AscW("}")
                            token.Type = RtfTokenType.GroupEnd
                            c = rtf.Read()
                        Case AscW("\")
                            parseKeyword(token)
                        Case Else
                            token.Type = RtfTokenType.Text
                            parseText(token)
                    End Select
                Else
                    'End of File
                    token.Type = RtfTokenType.Eof
                End If

                Return token
            End Function


#End Region

#Region "Private methods, described"

            ''' <summary>
            ''' Read a keyword in the RTF document.
            ''' </summary>
            ''' <param name="token"> RTF token that is assigned the keyword. </param>
            Private Sub parseKeyword(ByVal token As RtfToken)
                'He cleaned StringBuilders
                keysb.Length = 0
                parsb.Length = 0

                Dim parametroInt As Integer = 0
                Dim negativo As Boolean = False

                c = rtf.Read()

                'If the character read is not a letter -> This is a smbolo of Control or a special character:' \ \ ',' \ ( 'or' \) '
                If (Not Char.IsLetter(ChrW(c))) Then
                    If c = AscW("\") OrElse c = AscW("{") OrElse c = AscW("}") Then 'Special Character
                        token.Type = RtfTokenType.Text
                        token.Key = (ChrW(c)).ToString()
                    Else 'Control Symbol
                        token.Type = RtfTokenType.Control
                        token.Key = (ChrW(c)).ToString()

                        'If this is a special character (code of 8 bits) is read hexadecimal parmetro
                        If token.Key = "'" Then
                            Dim cod As String = ""

                            cod &= CChar(ChrW(rtf.Read()))
                            cod &= CChar(ChrW(rtf.Read()))

                            token.HasParameter = True

                            token.Parameter = Convert.ToInt32(cod, 16)
                        End If

                        'TODO: There mssmbolos with parmetros Control?
                    End If

                    c = rtf.Read()
                Else 'The character read is a letter
                    'We read the complete keyword (up alfanumrico not find a character, eg' \ '' '
                    Do While Char.IsLetter(ChrW(c))
                        keysb.Append(ChrW(c))

                        c = rtf.Read()
                    Loop

                    'Is assigned the keyword read
                    token.Type = RtfTokenType.Keyword
                    token.Key = keysb.ToString()

                    'Check if the keyword has parmetro
                    If Char.IsDigit(ChrW(c)) OrElse c = AscW("-") Then
                        token.HasParameter = True

                        'It is comprubea if negative parmetro
                        If c = AscW("-"c) Then
                            negativo = True

                            c = rtf.Read()
                        End If

                        'We read the full parmetro
                        Do While Char.IsDigit(ChrW(c))
                            parsb.Append(ChrW(c))

                            c = rtf.Read()
                        Loop

                        parametroInt = Convert.ToInt32(parsb.ToString())

                        If negativo Then
                            parametroInt = -parametroInt
                        End If

                        'Is assigned the keyword parmetro
                        token.Parameter = parametroInt
                    End If

                    If c = AscW(" "c) Then
                        c = rtf.Read()
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Read a string of RTF document.
            ''' </summary>
            ''' <param name="token"> RTF token that is assigned the keyword. </param>
            Private Sub parseText(ByVal token As RtfToken)
                'Clean the StringBuilder
                keysb.Length = 0

                Do While c <> AscW("\"c) AndAlso c <> AscW("}"c) AndAlso c <> AscW("{"c) AndAlso c <> Eof
                    keysb.Append(ChrW(c))

                    c = rtf.Read()

                    'It ignores the carriage returns, tabs and null characters
                    Do While c = Asc(ControlChars.Cr) OrElse c = Asc(ControlChars.Lf) OrElse c = Asc(ControlChars.Tab) OrElse c = Asc(ControlChars.NullChar)
                        c = rtf.Read()
                    Loop
                Loop

                token.Key = keysb.ToString()
            End Sub

#End Region
        End Class
    End Namespace
End Namespace
