'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfMerger
'* Description: Class to combine multiple RTF documents.
'* Notes: Originally contributed by Fabio Borghi.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Net.Sgoliver.NRtfTree.Util
Imports System.Text.RegularExpressions
Imports System.Runtime.InteropServices
Imports System.Drawing

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' Class to combine multiple RTF documents.
        ''' </summary>
        Public Class RtfMerger
            Private baseRtfDoc As RtfTree = Nothing
            Private destFilePath As String
            Private removeLastPar As Boolean

            Private placeHolder As Dictionary(Of String, RtfTree) = Nothing

            ''' <summary>
            ''' RtfMerger class constructor.
            ''' </summary>
            ''' <param name="sSourceDocFullPathName"> Route the document template. </param>
            ''' <param name="sDestFileFullPathName"> Route the resulting document. </param>
            ''' <param name="bolRemoveLastParCmd"> Indicates whether to delete the node ltimo \ pair of documents inserted into the template. </param>
            Public Sub New(ByVal sSourceDocFullPathName As String, ByVal sDestFileFullPathName As String, ByVal bolRemoveLastParCmd As Boolean)
                'It loads the document source
                baseRtfDoc = New RtfTree()
                baseRtfDoc.LoadRtfFile(sSourceDocFullPathName)

                'It saves the document's path destination
                destFilePath = sDestFileFullPathName

                'Indicative of ltimo eliminacin node \ par to inserted documents
                removeLastPar = bolRemoveLastParCmd

                'It creates the list of parmetros of sustitucin (placeholders)
                placeHolder = New Dictionary(Of String, RtfTree)()
            End Sub

            ''' <summary>
            ''' Associates a new parmetro of sustitucin (placeholder) with the path of the document to insert.
            ''' </summary>
            ''' <param name="ph"> Placeholder name. </param>
            ''' <param name="path"> Route the document to insert. </param>
            Public Sub AddPlaceHolder(ByVal ph As String, ByVal path As String)
                Dim tree As New RtfTree()

                Dim res As Integer = tree.LoadRtfFile(path)

                If res = 0 Then
                    placeHolder.Add(ph, tree)
                End If
            End Sub

            ''' <summary>
            ''' Disassociate a parmetro of sustitucin (placeholder) with the path of the document to insert.
            ''' </summary>
            ''' <param name="ph"> Placeholder name. </param>
            Public Sub RemovePlaceHolder(ByVal ph As String)
                placeHolder.Remove(ph)
            End Sub

            ''' <summary>
            ''' Performs combinacin RTF documents.
            ''' </summary>
            Public Sub MergeRtfDoc()
                'We get the primary group of the tree
                Dim parentNode As RtfTreeNode = baseRtfDoc.MainGroup

                'If the document is the main group
                If parentNode IsNot Nothing Then
                    'He analyzes the text of the document in search of replacement parmetros and combine documents
                    analizeTextContent(parentNode)

                    'It saves the resulting document
                    If destFilePath IsNot Nothing Then
                        baseRtfDoc.SaveRtf(destFilePath)
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Analyzes the text of the document in search of replacement parmetros and merge the documents.
            ''' </summary>
            ''' <param name="parentNode"> Node in the tree to be processed. </param>
            Private Sub analizeTextContent(ByVal parentNode As RtfTreeNode)
                Dim docToInsert As RtfTree = Nothing
                Dim indPH As Integer
                Dim iNdIndex As Integer

                'If the node is of type group and contains child nodes
                If parentNode IsNot Nothing AndAlso parentNode.HasChildNodes() Then
                    'You cover all child nodes
                    iNdIndex = 0
                    Do While iNdIndex < parentNode.ChildNodes.Count

                        'Current Node
                        Dim currNode As RtfTreeNode = parentNode.ChildNodes(iNdIndex)

                        'If the current node is of type text labels are looking to replace
                        If currNode.NodeType = RtfNodeType.Text Then
                            docToInsert = Nothing

                            'You cover all labels set
                            For Each ph As String In placeHolder.Keys
                                'He finds the next occurrence of the current label
                                indPH = currNode.NodeKey.IndexOf(ph)

                                'If you tag found
                                If indPH <> -1 Then
                                    'Recovering the tree to add in the current label
                                    docToInsert = placeHolder(ph).CloneTree()

                                    'New tree is inserted in the base tree
                                    mergeCore(parentNode, iNdIndex, docToInsert, ph, indPH)

                                    'As the current node may decrement the index has changed
                                    'and out of the loop to be analyzed again
                                    iNdIndex -= 1
                                    Exit For
                                End If
                            Next ph
                        Else
                            'If the current node has children nodes children are analyzed
                            If currNode.HasChildNodes() Then
                                analizeTextContent(currNode)
                            End If
                        End If

                        iNdIndex += 1
                    Loop
                End If
            End Sub

            ''' <summary>
            ''' Inserts a new tree in place of a text label of the tree base.
            ''' </summary>
            ''' <param name="parentNode"> Node type est group that is processed. </param>
            ''' <param name="iNdIndex"> index (within the parent group) node est text processing. </param>
            ''' <param name="docToInsert"> RTF to insert new tree. </param>
            ''' <param name="strCompletePlaceholder"> Text of the label to be replaced. </param>
            ''' <param name="intPlaceHolderNodePos"> Position of the label will be replaced within the text node est processing. </param>
            Private Sub mergeCore(ByVal parentNode As RtfTreeNode, ByVal iNdIndex As Integer, ByVal docToInsert As RtfTree, ByVal strCompletePlaceholder As String, ByVal intPlaceHolderNodePos As Integer)
                'If the document to insert non est Vaco
                If docToInsert.RootNode.HasChildNodes() = True Then
                    Dim currentIndex As Integer = iNdIndex + 1

                    'Are combined color table and adjust the colors of the document to add
                    mainAdjustColor(docToInsert)

                    'It combines the font tables and adjusting the source document to insert
                    mainAdjustFont(docToInsert)

                    'It removes the header information of the document to add (colors, fonts, info, ...)
                    cleanToInsertDoc(docToInsert)

                    'If the document is to add content
                    If docToInsert.RootNode.FirstChild.HasChildNodes() Then
                        'New document is inserted in the base tree
                        execMergeDoc(parentNode, docToInsert, currentIndex)
                    End If

                    'If the label does est at the end of the text node:
                    'This inserts a text node with the rest of the original (unlabeled)
                    If parentNode.ChildNodes(iNdIndex).NodeKey.Length <> (intPlaceHolderNodePos + strCompletePlaceholder.Length) Then
                        'This inserts a text node with the rest of the original (unlabeled)
                        Dim remText As String = parentNode.ChildNodes(iNdIndex).NodeKey.Substring(parentNode.ChildNodes(iNdIndex).NodeKey.IndexOf(strCompletePlaceholder) + strCompletePlaceholder.Length)

                        parentNode.InsertChild(currentIndex + 1, New RtfTreeNode(RtfNodeType.Text, remText, False, 0))
                    End If

                    'If the label was replaced early in the text node we remove the node
                    'original because it is not necessary
                    If intPlaceHolderNodePos = 0 Then
                        parentNode.RemoveChild(iNdIndex)
                        'Otherwise we substitute for the text before the tag
                    Else
                        parentNode.ChildNodes(iNdIndex).NodeKey = parentNode.ChildNodes(iNdIndex).NodeKey.Substring(0, intPlaceHolderNodePos)
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Get the code passed as the source parmetro, insertndola sources on the table if necessary.
            ''' </summary>
            ''' <param name="fontDestTbl"> Resulting table of sources. </param>
            ''' <param name="sFontName"> Source sought. </param>
            ''' <returns></returns>
            Private Function getFontID(ByRef fontDestTbl As RtfFontTable, ByVal sFontName As String) As Integer
                Dim iExistingFontID As Integer = -1

                iExistingFontID = fontDestTbl.IndexOf(sFontName)
                If iExistingFontID = -1 Then
                    fontDestTbl.AddFont(sFontName)
                    iExistingFontID = fontDestTbl.IndexOf(sFontName)

                    Dim nodeListToInsert As RtfNodeCollection = baseRtfDoc.RootNode.SelectNodes("fonttbl")

                    Dim ftFont As New RtfTreeNode(RtfNodeType.Group)
                    ftFont.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "f", True, iExistingFontID))
                    ftFont.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "fnil", False, 0))
                    ftFont.AppendChild(New RtfTreeNode(RtfNodeType.Text, sFontName & ";", False, 0))

                    nodeListToInsert(0).ParentNode.AppendChild(ftFont)
                End If

                Return iExistingFontID
            End Function

            ''' <summary>
            ''' Gets the color code would last as parmetro, insertndolo in the color table if necessary.
            ''' </summary>
            ''' <param name="colorDestTbl"> Resulting color table. </param>
            ''' <param name="iColorName"> Color searched. </param>
            ''' <returns></returns>
            Private Function getColorID(ByVal colorDestTbl As RtfColorTable, ByVal iColorName As Color) As Integer
                Dim iExistingColorID As Integer

                iExistingColorID = colorDestTbl.IndexOf(iColorName)
                If iExistingColorID = -1 Then
                    iExistingColorID = colorDestTbl.Count
                    colorDestTbl.AddColor(iColorName)

                    Dim nodeListToInsert As RtfNodeCollection = baseRtfDoc.RootNode.SelectNodes("colortbl")

                    nodeListToInsert(0).ParentNode.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "red", True, iColorName.R))
                    nodeListToInsert(0).ParentNode.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "green", True, iColorName.G))
                    nodeListToInsert(0).ParentNode.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "blue", True, iColorName.B))
                    nodeListToInsert(0).ParentNode.AppendChild(New RtfTreeNode(RtfNodeType.Text, ";", False, 0))
                End If

                Return iExistingColorID
            End Function

            ''' <summary>
            ''' Sets the source document to insert.
            ''' </summary>
            ''' <param name="docToInsert"> Document to be inserted. </param>
            Private Sub mainAdjustFont(ByVal docToInsert As RtfTree)
                Dim fontDestTbl As RtfFontTable = baseRtfDoc.GetFontTable()
                Dim fontToCopyTbl As RtfFontTable = docToInsert.GetFontTable()

                adjustFontRecursive(docToInsert.RootNode, fontDestTbl, fontToCopyTbl)
            End Sub

            ''' <summary>
            ''' Sets the source document to insert.
            ''' </summary>
            ''' <param name="parentNode"> Est node group to be processed. </param>
            ''' <param name="fontDestTbl"> Resulting table of sources. </param>
            ''' <param name="fontToCopyTbl"> Table of sources of document to be inserted. </param>
            Private Sub adjustFontRecursive(ByVal parentNode As RtfTreeNode, ByVal fontDestTbl As RtfFontTable, ByVal fontToCopyTbl As RtfFontTable)
                If parentNode IsNot Nothing AndAlso parentNode.HasChildNodes() Then
                    For iNdIndex As Integer = 0 To parentNode.ChildNodes.Count - 1
                        If parentNode.ChildNodes(iNdIndex).NodeType = RtfNodeType.Keyword AndAlso _
                            (parentNode.ChildNodes(iNdIndex).NodeKey = "f" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "stshfdbch" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "stshfloch" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "stshfhich" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "stshfbi" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "deff" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "af") AndAlso _
                            parentNode.ChildNodes(iNdIndex).HasParameter = True Then

                            parentNode.ChildNodes(iNdIndex).Parameter = getFontID(fontDestTbl, fontToCopyTbl(parentNode.ChildNodes(iNdIndex).Parameter))

                        End If

                        adjustFontRecursive(parentNode.ChildNodes(iNdIndex), fontDestTbl, fontToCopyTbl)
                    Next iNdIndex
                End If
            End Sub

            ''' <summary>
            ''' Adjust the colors of the document to insert.
            ''' </summary>
            ''' <param name="docToInsert"> Document to be inserted. </param>
            Private Sub mainAdjustColor(ByVal docToInsert As RtfTree)
                Dim colorDestTbl As RtfColorTable = baseRtfDoc.GetColorTable()
                Dim colorToCopyTbl As RtfColorTable = docToInsert.GetColorTable()

                adjustColorRecursive(docToInsert.RootNode, colorDestTbl, colorToCopyTbl)
            End Sub

            ''' <summary>
            ''' Adjust the colors of the document to insert.
            ''' </summary>
            ''' <param name="parentNode"> Est node group to be processed. </param>
            ''' <param name="colorDestTbl"> Resulting color table. </param>
            ''' <param name="colorToCopyTbl"> Color table the document to insert. </param>
            Private Sub adjustColorRecursive(ByVal parentNode As RtfTreeNode, ByVal colorDestTbl As RtfColorTable, ByVal colorToCopyTbl As RtfColorTable)
                If parentNode IsNot Nothing AndAlso parentNode.HasChildNodes() Then
                    For iNdIndex As Integer = 0 To parentNode.ChildNodes.Count - 1
                        If parentNode.ChildNodes(iNdIndex).NodeType = RtfNodeType.Keyword AndAlso _
                            (parentNode.ChildNodes(iNdIndex).NodeKey = "cf" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "cb" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "pncf" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "brdrcf" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "cfpat" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "cbpat" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "clcfpatraw" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "clcbpatraw" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "ulc" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "chcfpat" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "chcbpat" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "highlight" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "clcbpat" OrElse _
                             parentNode.ChildNodes(iNdIndex).NodeKey = "clcfpat") AndAlso _
                            parentNode.ChildNodes(iNdIndex).HasParameter = True Then

                            parentNode.ChildNodes(iNdIndex).Parameter = getColorID(colorDestTbl, colorToCopyTbl(parentNode.ChildNodes(iNdIndex).Parameter))

                        End If

                        adjustColorRecursive(parentNode.ChildNodes(iNdIndex), colorDestTbl, colorToCopyTbl)
                    Next iNdIndex
                End If
            End Sub

            ''' <summary>
            ''' Insert the new tree in the tree base (as a new group) removing all the document header inserted.
            ''' </summary>
            ''' <param name="parentNode"> Group base in which to insert the new tree. </param>
            ''' <param name="treeToCopyParent"> New tree to be inserted. </param>
            ''' <param name="intCurrIndex"> index in which to insert the new tree in the home group. </param>
            Private Sub execMergeDoc(ByVal parentNode As RtfTreeNode, ByVal treeToCopyParent As RtfTree, ByVal intCurrIndex As Integer)
                'He finds the first "\ pard" of the document (begin text)
                Dim nodePard As RtfTreeNode = treeToCopyParent.RootNode.FirstChild.SelectSingleChildNode("pard")

                'We get the index of the node within the main
                Dim indPard As Integer = treeToCopyParent.RootNode.FirstChild.ChildNodes.IndexOf(nodePard)

                'It creates the new group
                Dim newGroup As New RtfTreeNode(RtfNodeType.Group)

                'This paragraph shall reset options and source
                newGroup.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "pard", False, 0))
                newGroup.AppendChild(New RtfTreeNode(RtfNodeType.Keyword, "plain", False, 0))

                'Every child node is inserted in the new document in the document base
                For i As Integer = indPard + 1 To treeToCopyParent.RootNode.FirstChild.ChildNodes.Count - 1
                    Dim newNode As RtfTreeNode = treeToCopyParent.RootNode.FirstChild.ChildNodes(i).CloneNode(True)

                    newGroup.AppendChild(newNode)
                Next i

                'It inserting the new group with the new document
                parentNode.InsertChild(intCurrIndex, newGroup)
            End Sub

            ''' <summary>
            ''' Eliminate unwanted elements of the document to insert, for example the nodes "\ par" final.
            ''' </summary>
            ''' <param name="docToInsert"> Document to be inserted. </param>
            Private Sub cleanToInsertDoc(ByVal docToInsert As RtfTree)
                'Deletes the ltimo "\ par" the document if there
                Dim lastNode As RtfTreeNode = docToInsert.RootNode.FirstChild.LastChild

                If removeLastPar Then
                    If lastNode.NodeType = RtfNodeType.Keyword AndAlso lastNode.NodeKey = "par" Then
                        docToInsert.RootNode.FirstChild.RemoveChild(lastNode)
                    End If
                End If
            End Sub
        End Class
    End Namespace
End Namespace
