'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfNodeCollection
'* Description: Collection of nodes in a tree RTF.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' Collection of nodes in an RTF document.
        ''' </summary>
        Public Class RtfNodeCollection
            Inherits CollectionBase
#Region "method published"

            ''' <summary>
            ''' AADE a new node to the current Collection.
            ''' </summary>
            ''' <param name="node"> New node AAAD. </param>
            ''' <returns> Posicin in which the new node is inserted. </returns>
            Public Function Add(ByVal node As RtfTreeNode) As Integer
                InnerList.Add(node)

                Return (InnerList.Count - 1)
            End Function

            ''' <summary>
            ''' Insert a node in a Brand New in the Collection posicin determined.
            ''' </summary>
            ''' <param name="index"> Posicin in which to insert the node. </param>
            ''' <param name="node"> New node to insert. </param>
            Public Sub Insert(ByVal index As Integer, ByVal node As RtfTreeNode)
                InnerList.Insert(index, node)
            End Sub

            ''' <summary>
            ''' RtfNodeCollection Indexer class.
            ''' Returns the node that occupies the position 'index' in the Collection.
            ''' </summary>
            Default Public Property Item(ByVal index As Integer) As RtfTreeNode
                Get
                    Return CType(InnerList(index), RtfTreeNode)
                End Get
                Set(ByVal value As RtfTreeNode)
                    InnerList(index) = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the index of the node passed as parmetro within the node list of the Collection.
            ''' </summary>
            ''' <param name="node"> Node searching the Collection. </param>
            ''' <returns> Index searched node. Return the value -1 if the node is not found within the Collection. </returns>
            Public Function IndexOf(ByVal node As RtfTreeNode) As Integer
                Return InnerList.IndexOf(node)
            End Function

            ''' <summary>
            ''' Returns the index of the node passed as parmetro within the node list of the Collection.
            ''' </summary>
            ''' <param name="node"> Node searching the Collection. </param>
            ''' <param name="startIndex"> Posicin within the Collection from which to search. </param>
            ''' <returns> Index searched node. Return the value -1 if the node is not found within the Collection. </returns>
            Public Function IndexOf(ByVal node As RtfTreeNode, ByVal startIndex As Integer) As Integer
                Return InnerList.IndexOf(node, startIndex)
            End Function

            ''' <summary>
            ''' Returns the index of the first node of the Collection whose key is passed as the parmetro.
            ''' </summary>
            ''' <param name="key"> Key to search the Collection. </param>
            ''' <returns> Index searched node. Return the value -1 if the node is not found within the Collection. </returns>
            Public Function IndexOf(ByVal key As String) As Integer
                Dim intFoundAt As Integer = -1

                If InnerList.Count > 0 Then
                    For intIndex As Integer = 0 To InnerList.Count - 1
                        If (CType(InnerList(intIndex), RtfTreeNode)).NodeKey = key Then
                            intFoundAt = intIndex
                            Exit For
                        End If
                    Next intIndex
                End If

                Return intFoundAt
            End Function

            ''' <summary>
            ''' Returns the index of the first node of the Collection whose key is passed as the parmetro.
            ''' </summary>
            ''' <param name="key"> Key to search the Collection. </param>
            ''' <param name="startIndex"> Posicin within the Collection from which to search. </param>
            ''' <returns> Index searched node. Return the value -1 if the node is not found within the Collection. </returns>
            Public Function IndexOf(ByVal key As String, ByVal startIndex As Integer) As Integer
                Dim intFoundAt As Integer = -1

                If InnerList.Count > 0 Then
                    For intIndex As Integer = startIndex To InnerList.Count - 1
                        If (CType(InnerList(intIndex), RtfTreeNode)).NodeKey = key Then
                            intFoundAt = intIndex
                            Exit For
                        End If
                    Next intIndex
                End If

                Return intFoundAt
            End Function

            ''' <summary>
            ''' AADE at the end of the Collection a new list of nodes.
            ''' </summary>
            ''' <param name="collection"> New list of nodes AAAD the current Collection. </param>
            Public Sub AddRange(ByVal collection As RtfNodeCollection)
                InnerList.AddRange(collection)
            End Sub

            ''' <summary>
            ''' Deletes a set of nodes adjacent to the Collection.
            ''' </summary>
            ''' <param name="index"> Index of the first node of the assembly to remove. </param>
            ''' <param name="count"> Numbers of nodes to remove. </param>
            Public Sub RemoveRange(ByVal index As Integer, ByVal count As Integer)
                InnerList.RemoveRange(index, count)
            End Sub

#End Region
        End Class
    End Namespace
End Namespace
