'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfReader
'* Description: RTF sequential analyzer.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.IO

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' This class provides the necessary alternative methods of charging and sequential anlisis an RTF document.
        ''' </summary>
        Public Class RtfReader
#Region "Attributes private"

            Private rtf As TextReader 'File / Input string RTF
            Private lex As RtfLex  'lxico RTF Parser
            Private tok As RtfToken 'current Token
            Private reader As SarParser 'Rtf Reader

#End Region

#Region "Constructors"

            ''' <summary>
            ''' RtfReader class constructor.
            ''' </summary>
            ''' <param name="reader">
            ''' Object type containing the methods, described SARParser necessary for the treatment of
            ''' various elements of an RTF document.
            ''' </param>
            Public Sub New(ByVal reader As SarParser)
                'Initializing Default
                'lex = null;
                'tok = null;
                'rtf = null;

                Me.reader = reader
            End Sub

#End Region

#Region "alternative methods of Públicos"

            ''' <summary>
            ''' Load an RTF document because the file path that contains it.
            ''' </summary>
            ''' <param name="path"> Path of the file containing the RTF document. </param>
            ''' <returns>
            ''' Results loading the document. If the load is successful
            ''' returns the value 0.
            ''' </returns>
            Public Function LoadRtfFile(ByVal path As String) As Integer
                'Result of load
                Dim res As Integer = 0

                'It opens the input file
                rtf = New StreamReader(path)

                'It creates the RTF parser lxico
                lex = New RtfLex(rtf)

                'It returns the result of the load
                Return res
            End Function

            ''' <summary>
            ''' Load an RTF document given the character string that contains it.
            ''' </summary>
            ''' <param name="text"> Caractres string containing the RTF document. </param>
            ''' <returns>
            ''' Results loading the document. If the load is successful
            ''' returns the value 0.
            ''' </returns>
            Public Function LoadRtfText(ByVal text As String) As Integer
                'Result of load
                Dim res As Integer = 0

                'It opens the input file
                rtf = New StringReader(text)

                'It creates the RTF parser lxico
                lex = New RtfLex(rtf)

                'It returns the result of the load
                Return res
            End Function

            ''' <summary>
            ''' Starts anlisis RTF document and causes the call to the various alternative methods of
            ''' IRtfReader indicated object in the constructor of the class.
            ''' </summary>
            ''' <returns>
            ''' Anlisis result of the document. If the load is successful
            ''' returns the value 0.
            ''' </returns>
            Public Function Parse() As Integer
                'Result of anlisis
                Dim res As Integer = 0

                'Start the document
                reader.StartRtfDocument()

                'We get the first token
                tok = lex.NextToken()

                Do While tok.Type <> RtfTokenType.Eof
                    Select Case tok.Type
                        Case RtfTokenType.GroupStart
                            reader.StartRtfGroup()
                        Case RtfTokenType.GroupEnd
                            reader.EndRtfGroup()
                        Case RtfTokenType.Keyword
                            reader.RtfKeyword(tok.Key, tok.HasParameter, tok.Parameter)
                        Case RtfTokenType.Control
                            reader.RtfControl(tok.Key, tok.HasParameter, tok.Parameter)
                        Case RtfTokenType.Text
                            reader.RtfText(tok.Key)
                        Case Else
                            res = -1
                    End Select

                    'We get the next token
                    tok = lex.NextToken()
                Loop

                'Finish the document
                reader.EndRtfDocument()

                'Close the stream
                rtf.Close()

                Return res
            End Function

#End Region
        End Class
    End Namespace
End Namespace
