'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfStyleSheet
'* Description: Represents a stylesheet from an RTF document.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports Net.Sgoliver.NRtfTree.Core

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' Represents a style sheet in an RTF document
        ''' </summary>
        Public Class RtfStyleSheet

#Region "Private Attributes"

            Private m_index As Integer = 0
            Private m_name As String = ""
            Private m_type As RtfStyleSheetType = RtfStyleSheetType.Paragraph
            Private m_additive As Boolean = False
            Private m_basedOn As Integer = -1
            Private m_next As Integer = -1
            Private m_autoUpdate As Boolean = False
            Private m_hidden As Boolean = False
            Private m_link As Integer = -1
            Private m_locked As Boolean = False
            Private m_personal As Boolean = False
            Private m_compose As Boolean = False
            Private m_reply As Boolean = False
            Private m_styrsid As Integer = -1
            Private m_semiHidden As Boolean = False
            Private m_keyCode As RtfNodeCollection = Nothing
            Private m_formatting As RtfNodeCollection = Nothing

#End Region

#Region "Public Properties"

            ''' <summary>
            ''' Index stylesheet
            ''' </summary>
            Public Property Index() As Integer
                Get
                    Return m_index
                End Get
                Set(ByVal value As Integer)
                    m_index = value
                End Set
            End Property

            ''' <summary>
            ''' Name of the stylesheet.
            ''' </summary>
            Public Property Name() As String
                Get
                    Return m_name
                End Get
                Set(ByVal value As String)
                    m_name = value
                End Set
            End Property

            ''' <summary>
            ''' Type style sheet.
            ''' </summary>
            Public Property Type() As RtfStyleSheetType
                Get
                    Return m_type
                End Get
                Set(ByVal value As RtfStyleSheetType)
                    m_type = value
                End Set
            End Property

            ''' <summary>
            ''' In the style sheets of character type indicates whether the specified style is to be added to the current paragraph style, instead of completely overwriting the current style.
            ''' </summary>
            Public Property Additive() As Boolean
                Get
                    Return m_additive
                End Get
                Set(ByVal value As Boolean)
                    m_additive = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates the style in which it is based on current style.
            ''' </summary>
            Public Property BasedOn() As Integer
                Get
                    Return m_basedOn
                End Get
                Set(ByVal value As Integer)
                    m_basedOn = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates the style used in the next paragraph.
            ''' </summary>
            Public Property [Next]() As Integer
                Get
                    Return m_next
                End Get
                Set(ByVal value As Integer)
                    m_next = value
                End Set
            End Property

            ''' <summary>
            ''' Indicate whether the styles are automatically updated.
            ''' </summary>
            Public Property AutoUpdate() As Boolean
                Get
                    Return m_autoUpdate
                End Get
                Set(ByVal value As Boolean)
                    m_autoUpdate = value
                End Set
            End Property

            ''' <summary>
            ''' It states that the current style does not appear in the drop-down lists of styles.
            ''' </summary>
            Public Property Hidden() As Boolean
                Get
                    Return m_hidden
                End Get
                Set(ByVal value As Boolean)
                    m_hidden = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates the style that is bound at present.
            ''' </summary>
            Public Property Link() As Integer
                Get
                    Return m_link
                End Get
                Set(ByVal value As Integer)
                    m_link = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates that the current style is locked.
            ''' </summary>
            Public Property Locked() As Boolean
                Get
                    Return m_locked
                End Get
                Set(ByVal value As Boolean)
                    m_locked = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates that the current style is the style of personal email.
            ''' </summary>
            Public Property Personal() As Boolean
                Get
                    Return m_personal
                End Get
                Set(ByVal value As Boolean)
                    m_personal = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates that the current style is the style of composition of e-mail.
            ''' </summary>
            Public Property Compose() As Boolean
                Get
                    Return m_compose
                End Get
                Set(ByVal value As Boolean)
                    m_compose = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates that the current style is the style of e-mail response.
            ''' </summary>
            Public Property Reply() As Boolean
                Get
                    Return m_reply
                End Get
                Set(ByVal value As Boolean)
                    m_reply = value
                End Set
            End Property

            ''' <summary>
            ''' Rsity Tied to the table, N is the rsity of the author who implemented the style.
            ''' </summary>
            Public Property Styrsid() As Integer
                Get
                    Return m_styrsid
                End Get
                Set(ByVal value As Integer)
                    m_styrsid = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates that style does not appear in the dropdown menus.
            ''' </summary>
            Public Property SemiHidden() As Boolean
                Get
                    Return m_semiHidden
                End Get
                Set(ByVal value As Boolean)
                    m_semiHidden = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates the hotkey to set the current style.
            ''' </summary>
            Public Property KeyCode() As RtfNodeCollection
                Get
                    Return m_keyCode
                End Get
                Set(ByVal value As RtfNodeCollection)
                    m_keyCode = value
                End Set
            End Property

            ''' <summary>
            ''' Format options contained in the current style.
            ''' </summary>
            Public Property Formatting() As RtfNodeCollection
                Get
                    Return m_formatting
                End Get
                Set(ByVal value As RtfNodeCollection)
                    m_formatting = value
                End Set
            End Property

#End Region

#Region "Constructors"

            ''' <summary>
            ''' RtfStyleSheet class constructor.
            ''' </summary>
            Public Sub New()
                m_keyCode = Nothing
                m_formatting = New RtfNodeCollection()
            End Sub

#End Region

        End Class
    End Namespace
End Namespace
