'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfStyleSheetTable
'* Description: Represents the table stylesheets an RTF document.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Collections
Imports Net.Sgoliver.NRtfTree.Core

Namespace Net.Sgoliver.NRtfTree
    Namespace Util
        ''' <summary>
        ''' Represents the table styles in a RTF document.
        ''' </summary>
        Public Class RtfStyleSheetTable
            Private stylesheets As Dictionary(Of Integer, RtfStyleSheet) = Nothing

            ''' <summary>
            ''' Constructor for the table styles.
            ''' </summary>
            Public Sub New()
                stylesheets = New Dictionary(Of Integer, RtfStyleSheet)()
            End Sub

            ''' <summary>
            ''' Add a new style to the table styles. The style is added with a new index does not exist in the table.
            ''' </summary>
            ''' <param name="ss"> New style to add to the table. </param>
            Public Sub AddStyleSheet(ByVal ss As RtfStyleSheet)
                ss.Index = newStyleSheetIndex()

                stylesheets.Add(ss.Index, ss)
            End Sub

            ''' <summary>
            ''' Add a new style to the table styles. The style is added to the index in the parameter style.
            ''' </summary>
            ''' <param name="index"> Index to add style to the table. </param>
            ''' <param name="ss"> New style to add to the table. </param>
            Public Sub AddStyleSheet(ByVal index As Integer, ByVal ss As RtfStyleSheet)
                ss.Index = index

                stylesheets.Add(index, ss)
            End Sub

            ''' <summary>
            ''' Removes a style from the styles table by index.
            ''' </summary>
            ''' <param name="index"></param>
            Public Sub RemoveStyleSheet(ByVal index As Integer)
                stylesheets.Remove(index)
            End Sub

            ''' <summary>
            ''' Remove from the table styles in the parameter style.
            ''' </summary>
            ''' <param name="ss"></param>
            Public Sub RemoveStyleSheet(ByVal ss As RtfStyleSheet)
                stylesheets.Remove(ss.Index)
            End Sub

            ''' <summary>
            ''' Retrieves a style from the styles table by index.
            ''' </summary>
            ''' <param name="index"> Style index to retrieve. </param>
            ''' <returns> Style whose index is passed as parameter. </returns>
            Public Function GetStyleSheet(ByVal index As Integer) As RtfStyleSheet
                Return stylesheets(index)
            End Function

            ''' <summary>
            ''' Retrieves a style from the styles table by index.
            ''' </summary>
            ''' <param name="index"> Style index to retrieve. </param>
            ''' <returns> Style whose index is passed as parameter. </returns>
            Default Public ReadOnly Property Item(ByVal index As Integer) As RtfStyleSheet
                Get
                    Return stylesheets(index)
                End Get
            End Property

            ''' <summary>
            ''' Number of styles contained in the table styles.
            ''' </summary>
            Public ReadOnly Property Count() As Integer
                Get
                    Return stylesheets.Count
                End Get
            End Property

            ''' <summary>
            ''' Index style whose name is passed as parameter.
            ''' </summary>
            ''' <param name="name"> Style Name searched. </param>
            ''' <returns> Style whose name is passed as parameter. </returns>
            Public Function IndexOf(ByVal name As String) As Integer
                Dim intIndex As Integer = -1
                Dim fntIndex As IEnumerator = stylesheets.GetEnumerator()

                fntIndex.Reset()
                Do While fntIndex.MoveNext() = True
                    If (CType(fntIndex.Current, KeyValuePair(Of Integer, RtfStyleSheet))).Value.Name.Equals(name) Then
                        intIndex = System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, RtfStyleSheet))).Key)
                        Exit Do
                    End If
                Loop

                Return intIndex
            End Function

            ''' <summary>
            ''' Calculate a new index to insert a table style.
            ''' </summary>
            ''' <returns> Index to add the next style. </returns>
            Private Function newStyleSheetIndex() As Integer
                Dim intIndex As Integer = -1
                Dim fntIndex As IEnumerator = stylesheets.GetEnumerator()

                fntIndex.Reset()
                Do While fntIndex.MoveNext() = True
                    If System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, RtfStyleSheet))).Key) > intIndex Then
                        intIndex = System.Convert.ToInt32((CType(fntIndex.Current, KeyValuePair(Of Integer, RtfStyleSheet))).Key)
                    End If
                Loop

                Return (intIndex + 1)
            End Function
        End Class
    End Namespace
End Namespace
