'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfTree
'* Description: Represents an RTF document as a tree.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.IO
Imports System.Text
Imports System.Drawing
Imports Net.Sgoliver.NRtfTree.Util

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' Reresenta the tree-shaped structure of an RTF document.
        ''' </summary>
        Public Class RtfTree
#Region "Attributes private"

            ''' <summary>
            ''' Node raz RTF document.
            ''' </summary>
            Private m_RootNode As RtfTreeNode
            ''' <summary>
            ''' File / Input string RTF
            ''' </summary>
            Private  m_rtf As TextReader
            ''' <summary>
            ''' Lxico RTF Parser
            ''' </summary>
            Private lex As RtfLex
            ''' <summary>
            ''' Current Token
            ''' </summary>
            Private tok As RtfToken
            ''' <summary>
            ''' Depth of the current node
            ''' </summary>
            Private level As Integer
            ''' <summary>
            ''' Indicates whether to decode the special characters (\ ') unindolos to contiguous text nodes.
            ''' </summary>
            Private  m_mergeSpecialCharacters As Boolean

#End Region

#Region "constructors"

            ''' <summary>
            ''' RtfTree class constructor.
            ''' </summary>
            Public Sub New()
                'It creates the document node raz
                m_RootNode = New RtfTreeNode(RtfNodeType.Root, "ROOT", False, 0)

                m_RootNode.Tree = Me

                'Initializing Default

                'MergeSpecialCharacters property is initialized
                 m_mergeSpecialCharacters = False

                'It initializes the current depth
                'level = 0;
            End Sub

#End Region

#Region "alternative methods of Públicos"

            ''' <summary>
            ''' Make an exact copy of the tree RTF.
            ''' </summary>
            ''' <returns> Returns an exact copy of the tree RTF. </returns>
            Public Function CloneTree() As RtfTree
                Dim clon As New RtfTree()

                clon.RootNode = Me.m_RootNode.CloneNode(True)

                Return clon
            End Function

            ''' <summary>
            ''' Upload a file in RTF format.
            ''' </summary>
            ''' <param name="path"> File path to the document. </param>
            ''' <returns> It returns the value 0 if any error did not occur in loading the document.
            ''' Otherwise it returns -1. </returns>
            Public Function LoadRtfFile(ByVal path As String) As Integer
                'Result of load
                Dim res As Integer = 0

                'It opens the input file
                 m_rtf = New StreamReader(path)

                'It creates the RTF parser lxico
                lex = New RtfLex( m_rtf)

                'The tree is loaded with the contents of the RTF document
                res = parseRtfTree()

                'Close the stream
                 m_rtf.Close()

                'It returns the result of the load
                Return res
            End Function

            ''' <summary>
            ''' Loads a string of text with RTF.
            ''' </summary>
            ''' <param name="text"> Text string containing the document. </param>
            ''' <returns> It returns the value 0 if any error did not occur in loading the document.
            ''' Otherwise it returns -1. </returns>
            Public Function LoadRtfText(ByVal text As String) As Integer
                'Result of load
                Dim res As Integer = 0

                'It opens the input file
                 m_rtf = New StringReader(text)

                'It creates the RTF parser lxico
                lex = New RtfLex( m_rtf)

                'The tree is loaded with the contents of the RTF document
                res = parseRtfTree()

                'Close the stream
                 m_rtf.Close()

                'It returns the result of the load
                Return res
            End Function

            ''' <summary>
            ''' Enter the code RTF document to a file.
            ''' </summary>
            ''' <param name="filePath"> Path of file to generate the RTF document. </param>
            Public Sub SaveRtf(ByVal filePath As String)
                'Stream output
                Dim sw As New StreamWriter(filePath)

                'It transforms the tree RTF to text and written to the file
                sw.Write(Me.RootNode.Rtf)

                'Close the file
                sw.Flush()
                sw.Close()
            End Sub

            ''' <summary>
            ''' Textual representacin returns a document loaded.
            ''' </summary>
            ''' <returns> String with the representacin the document. </returns>
            Public Overrides Function ToString() As String
                Dim res As String = ""

                res = toStringInm(m_RootNode, 0, False)

                Return res
            End Function

            ''' <summary>
            ''' Textual representacin returns a document loaded. AADE the node type to the left of the node content.
            ''' </summary>
            ''' <returns> String with the representacin the document. </returns>
            Public Function ToStringEx() As String
                Dim res As String = ""

                res = toStringInm(m_RootNode, 0, True)

                Return res
            End Function

            ''' <summary>
            ''' Returns the table of sources of RTF document.
            ''' </summary>
            ''' <returns> Table RTF document sources </returns>
            Public Function GetFontTable() As RtfFontTable
                Dim tablaFuentes As New RtfFontTable()

                'Root node of the document
                Dim root As RtfTreeNode = Me.m_RootNode

                'Group of the document
                Dim nprin As RtfTreeNode = root.FirstChild

                'We seek the source table in the tree
                Dim enc As Boolean = False
                Dim i As Integer = 0
                Dim Ntf As New RtfTreeNode() 'Node with the table of sources

                Do While (Not enc) AndAlso i < nprin.ChildNodes.Count
                    If nprin.ChildNodes(i).NodeType = RtfNodeType.Group AndAlso nprin.ChildNodes(i).FirstChild.NodeKey = "fonttbl" Then
                        enc = True
                        Ntf = nprin.ChildNodes(i)
                    End If

                    i += 1
                Loop

                'Fill the array of sources
                For j As Integer = 1 To Ntf.ChildNodes.Count - 1
                    Dim fuente As RtfTreeNode = Ntf.ChildNodes(j)

                    Dim indiceFuente As Integer = -1
                    Dim nombreFuente As String = Nothing

                    For Each nodo As RtfTreeNode In fuente.ChildNodes
                        If nodo.NodeKey = "f" Then
                            indiceFuente = nodo.Parameter
                        End If

                        If nodo.NodeType = RtfNodeType.Text Then
                            nombreFuente = nodo.NodeKey.Substring(0, nodo.NodeKey.Length - 1)
                        End If
                    Next nodo

                    tablaFuentes.AddFont(indiceFuente, nombreFuente)
                Next j

                Return tablaFuentes
            End Function

            ''' <summary>
            ''' Returns the color table RTF document.
            ''' </summary>
            ''' <returns> Color table RTF document </returns>
            Public Function GetColorTable() As RtfColorTable
                Dim tablaColores As New RtfColorTable()

                'Root node of the document
                Dim root As RtfTreeNode = Me.m_RootNode

                'Group of the document
                Dim nprin As RtfTreeNode = root.FirstChild

                'We the color table in the tree
                Dim enc As Boolean = False
                Dim i As Integer = 0
                Dim Ntc As New RtfTreeNode() 'Node with the table of sources

                Do While (Not enc) AndAlso i < nprin.ChildNodes.Count
                    If nprin.ChildNodes(i).NodeType = RtfNodeType.Group AndAlso nprin.ChildNodes(i).FirstChild.NodeKey = "colortbl" Then
                        enc = True
                        Ntc = nprin.ChildNodes(i)
                    End If

                    i += 1
                Loop

                'Fill the array of colors
                Dim rojo As Integer = 0
                Dim verde As Integer = 0
                Dim azul As Integer = 0

                'Aadimos the default color, in this case black.
                'tabla.Add (Color.FromArgb (red, green, blue));

                For j As Integer = 1 To Ntc.ChildNodes.Count - 1
                    Dim nodo As RtfTreeNode = Ntc.ChildNodes(j)

                    If nodo.NodeType = RtfNodeType.Text AndAlso nodo.NodeKey.Trim() = ";" Then
                        tablaColores.AddColor(Color.FromArgb(rojo, verde, azul))

                        rojo = 0
                        verde = 0
                        azul = 0
                    ElseIf nodo.NodeType = RtfNodeType.Keyword Then
                        Select Case nodo.NodeKey
                            Case "red"
                                rojo = nodo.Parameter
                            Case "green"
                                verde = nodo.Parameter
                            Case "blue"
                                azul = nodo.Parameter
                        End Select
                    End If
                Next j

                Return tablaColores
            End Function

            ''' <summary>
            ''' Returns the table style sheets RTF document.
            ''' </summary>
            ''' <returns> Chart style sheets RTF document. </returns>
            Public Function GetStyleSheetTable() As RtfStyleSheetTable
                Dim sstable As New RtfStyleSheetTable()

                Dim sst As RtfTreeNode = MainGroup.SelectSingleGroup("stylesheet")

                Dim styles As RtfNodeCollection = sst.ChildNodes

                For i As Integer = 1 To styles.Count - 1
                    Dim style As RtfTreeNode = styles(i)

                    Dim rtfss As RtfStyleSheet = ParseStyleSheet(style)

                    sstable.AddStyleSheet(rtfss.Index, rtfss)
                Next i

                Return sstable
            End Function

            ''' <summary>
            ''' Returns the information contained in the "\ info" in the RTF document.
            ''' </summary>
            ''' <returns> InfoGroup object with information of the group "\ info" in the RTF document. </returns>
            Public Function GetInfoGroup() As InfoGroup
                Dim info As InfoGroup = Nothing

                Dim infoNode As RtfTreeNode = Me.RootNode.SelectSingleNode("info")

                'If there is a node "\ info" exra all the information.
                If infoNode IsNot Nothing Then
                    Dim auxnode As RtfTreeNode = Nothing

                    info = New InfoGroup()

                    'Title
                    auxnode = Me.m_RootNode.SelectSingleNode("title")
                    If auxnode IsNot Nothing Then
                        info.Title = auxnode.NextSibling.NodeKey
                    End If

                    'Subject
                    auxnode = Me.m_RootNode.SelectSingleNode("subject")
                    If auxnode IsNot Nothing Then
                        info.Subject = auxnode.NextSibling.NodeKey
                    End If

                    'Author
                    auxnode = Me.m_RootNode.SelectSingleNode("author")
                    If auxnode IsNot Nothing Then
                        info.Author = auxnode.NextSibling.NodeKey
                    End If

                    'Manager
                    auxnode = Me.m_RootNode.SelectSingleNode("manager")
                    If auxnode IsNot Nothing Then
                        info.Manager = auxnode.NextSibling.NodeKey
                    End If

                    'Company
                    auxnode = Me.m_RootNode.SelectSingleNode("company")
                    If auxnode IsNot Nothing Then
                        info.Company = auxnode.NextSibling.NodeKey
                    End If

                    'Operator
                    auxnode = Me.m_RootNode.SelectSingleNode("operator")
                    If auxnode IsNot Nothing Then
                        info.Operator = auxnode.NextSibling.NodeKey
                    End If

                    'Category
                    auxnode = Me.m_RootNode.SelectSingleNode("category")
                    If auxnode IsNot Nothing Then
                        info.Category = auxnode.NextSibling.NodeKey
                    End If

                    'Keywords
                    auxnode = Me.m_RootNode.SelectSingleNode("keywords")
                    If auxnode IsNot Nothing Then
                        info.Keywords = auxnode.NextSibling.NodeKey
                    End If

                    'Comments
                    auxnode = Me.m_RootNode.SelectSingleNode("comment")
                    If auxnode IsNot Nothing Then
                        info.Comment = auxnode.NextSibling.NodeKey
                    End If

                    'Document comments
                    auxnode = Me.m_RootNode.SelectSingleNode("doccomm")
                    If auxnode IsNot Nothing Then
                        info.DocComment = auxnode.NextSibling.NodeKey
                    End If

                    'Hlinkbase (The base address that is used for the path of all relative hyperlinks inserted in the document)
                    auxnode = Me.m_RootNode.SelectSingleNode("hlinkbase")
                    If auxnode IsNot Nothing Then
                        info.HlinkBase = auxnode.NextSibling.NodeKey
                    End If

                    'Version
                    auxnode = Me.m_RootNode.SelectSingleNode("version")
                    If auxnode IsNot Nothing Then
                        info.Version = auxnode.Parameter
                    End If

                    'Internal Version
                    auxnode = Me.m_RootNode.SelectSingleNode("vern")
                    If auxnode IsNot Nothing Then
                        info.InternalVersion = auxnode.Parameter
                    End If

                    'Editing Time
                    auxnode = Me.m_RootNode.SelectSingleNode("edmins")
                    If auxnode IsNot Nothing Then
                        info.EditingTime = auxnode.Parameter
                    End If

                    'Number of Pages
                    auxnode = Me.m_RootNode.SelectSingleNode("nofpages")
                    If auxnode IsNot Nothing Then
                        info.NumberOfPages = auxnode.Parameter
                    End If

                    'Number of Chars
                    auxnode = Me.m_RootNode.SelectSingleNode("nofchars")
                    If auxnode IsNot Nothing Then
                        info.NumberOfChars = auxnode.Parameter
                    End If

                    'Number of Words
                    auxnode = Me.m_RootNode.SelectSingleNode("nofwords")
                    If auxnode IsNot Nothing Then
                        info.NumberOfWords = auxnode.Parameter
                    End If

                    'Id
                    auxnode = Me.m_RootNode.SelectSingleNode("id")
                    If auxnode IsNot Nothing Then
                        info.Id = auxnode.Parameter
                    End If

                    'Creation DateTime
                    auxnode = Me.m_RootNode.SelectSingleNode("creatim")
                    If auxnode IsNot Nothing Then
                        info.CreationTime = parseDateTime(auxnode.ParentNode)
                    End If

                    'Revision DateTime
                    auxnode = Me.m_RootNode.SelectSingleNode("revtim")
                    If auxnode IsNot Nothing Then
                        info.RevisionTime = parseDateTime(auxnode.ParentNode)
                    End If

                    'Last Print Time
                    auxnode = Me.m_RootNode.SelectSingleNode("printim")
                    If auxnode IsNot Nothing Then
                        info.LastPrintTime = parseDateTime(auxnode.ParentNode)
                    End If

                    'Backup Time
                    auxnode = Me.m_RootNode.SelectSingleNode("buptim")
                    If auxnode IsNot Nothing Then
                        info.BackupTime = parseDateTime(auxnode.ParentNode)
                    End If
                End If

                Return info
            End Function

            ''' <summary>
            ''' Returns the table with which Cdigo est codified RTF document.
            ''' </summary>
            ''' <returns> Table code would RTF document. If not specified in the document is returned Cdigo table current system. </returns>
            Public Function GetEncoding() As Encoding
                'Contributed by Jan Stuchlk

                Dim encoding As Encoding = encoding.Default

                Dim cpNode As RtfTreeNode = RootNode.SelectSingleNode("ansicpg")

                If cpNode IsNot Nothing Then
                    encoding = encoding.GetEncoding(cpNode.Parameter)
                End If

                Return encoding
            End Function

#End Region

#Region "Private methods, described"

            ''' <summary>
            ''' Parses the document and load it with tree structure.
            ''' </summary>
            ''' <returns> It returns the value 0 if any error did not occur in loading the document.
            ''' Otherwise it returns -1. </returns>
            Private Function parseRtfTree() As Integer
                'Result of loading the document
                Dim res As Integer = 0

                'Encoding default document
                Dim encoding As Encoding = encoding.Default

                'Current Node
                Dim curNode As RtfTreeNode = m_RootNode

                'New nodes to build the tree RTF
                Dim newNode As RtfTreeNode = Nothing

                'We get the first token
                tok = lex.NextToken()

                Do While tok.Type <> RtfTokenType.Eof
                    Select Case tok.Type
                        Case RtfTokenType.GroupStart
                            newNode = New RtfTreeNode(RtfNodeType.Group, "GROUP", False, 0)
                            curNode.AppendChild(newNode)
                            curNode = newNode
                            level += 1
                        Case RtfTokenType.GroupEnd
                            curNode = curNode.ParentNode
                            level -= 1
                        Case RtfTokenType.Keyword, RtfTokenType.Control, RtfTokenType.Text
                            If  m_mergeSpecialCharacters Then
                                'Contributed by Jan Stuchlk
                                Dim ISTEXT As Boolean = tok.Type = RtfTokenType.Text OrElse (tok.Type = RtfTokenType.Control AndAlso tok.Key = "'")
                                If curNode.LastChild IsNot Nothing AndAlso (curNode.LastChild.NodeType = RtfNodeType.Text AndAlso ISTEXT) Then
                                    If tok.Type = RtfTokenType.Text Then
                                        curNode.LastChild.NodeKey += tok.Key
                                        Exit Select
                                    End If
                                    If tok.Type = RtfTokenType.Control AndAlso tok.Key = "'" Then
                                        curNode.LastChild.NodeKey += DecodeControlChar(tok.Parameter, encoding)
                                        Exit Select
                                    End If
                                Else
                                    'First special character \'
                                    If tok.Type = RtfTokenType.Control AndAlso tok.Key = "'" Then
                                        newNode = New RtfTreeNode(RtfNodeType.Text, DecodeControlChar(tok.Parameter, encoding), False, 0)
                                        curNode.AppendChild(newNode)
                                        Exit Select
                                    End If
                                End If
                            End If

                            newNode = New RtfTreeNode(tok)
                            curNode.AppendChild(newNode)

                            If  m_mergeSpecialCharacters Then
                                'Contributed by Jan Stuchlk
                                If level = 1 AndAlso newNode.NodeType = RtfNodeType.Keyword AndAlso newNode.NodeKey = "ansicpg" Then
                                    encoding = encoding.GetEncoding(newNode.Parameter)
                                End If
                            End If

                        Case Else
                            res = -1
                    End Select

                    'We get the next token
                    tok = lex.NextToken()
                Loop

                'If the current level is not 0 (== some groups are not well trained est)
                If level <> 0 Then
                    res = -1
                End If

                'It returns the result of the load
                Return res
            End Function

            ''' <summary>
            ''' Decodes a special character indicated by decimal Cdigo
            ''' </summary>
            ''' <param name="code"> Cdigo the special character (\ ') </param>
            ''' <param name="enc"> Encoding used to decode the special character. </param>
            ''' <returns> Special character decoding. </returns>
            Private Shared Function DecodeControlChar(ByVal code As Integer, ByVal enc As Encoding) As String
                'Contributed by Jan Stuchlk
                Return enc.GetString(New Byte() {CByte(code)})
            End Function

            ''' <summary>
            ''' Assistant to generate alternative methods of Textual representacin RTF document.
            ''' </summary>
            ''' <param name="curNode"> Current node tree. </param>
            ''' <param name="level"> Current level in tree. </param>
            ''' <param name="showNodeTypes"> Indicates whether to show the type of each node in the tree. </param>
            ''' <returns> Textual Representacin node 'curNode' to level 'level' </returns>
            Private Function toStringInm(ByVal curNode As RtfTreeNode, ByVal level As Integer, ByVal showNodeTypes As Boolean) As String
                Dim res As New StringBuilder()

                Dim children As RtfNodeCollection = curNode.ChildNodes

                For i As Integer = 0 To level - 1
                    res.Append("  ")
                Next i

                If curNode.NodeType = RtfNodeType.Root Then
                    res.Append("ROOT" & Constants.vbCrLf)
                ElseIf curNode.NodeType = RtfNodeType.Group Then
                    res.Append("GROUP" & Constants.vbCrLf)
                Else
                    If showNodeTypes Then
                        res.Append(curNode.NodeType.ToString())
                        res.Append(": ")
                    End If

                    res.Append(curNode.NodeKey)

                    If curNode.HasParameter Then
                        res.Append(" ")
                        res.Append(Convert.ToString(curNode.Parameter))
                    End If

                    res.Append(Constants.vbCrLf)
                End If

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        res.Append(toStringInm(node, level + 1, showNodeTypes))
                    Next node
                End If

                Return res.ToString()
            End Function

            ''' <summary>
            ''' Parsing a date format "\ yr2005 \ mo12 \ dy2 \ HR22 \ min56 \ SEC15"
            ''' </summary>
            ''' <param name="group"> RTF Group date. </param>
            ''' <returns> DateTime object with the date read. </returns>
            Private Shared Function parseDateTime(ByVal group As RtfTreeNode) As DateTime
                Dim dt As DateTime

                Dim year As Integer = 0, month As Integer = 0, day As Integer = 0, hour As Integer = 0, min As Integer = 0, sec As Integer = 0

                For Each node As RtfTreeNode In group.ChildNodes
                    Select Case node.NodeKey
                        Case "yr"
                            year = node.Parameter
                        Case "mo"
                            month = node.Parameter
                        Case "dy"
                            day = node.Parameter
                        Case "hr"
                            hour = node.Parameter
                        Case "min"
                            min = node.Parameter
                        Case "sec"
                            sec = node.Parameter
                    End Select
                Next node

                dt = New DateTime(year, month, day, hour, min, sec)

                Return dt
            End Function

            ''' <summary>
            ''' Extract the text of a tree RTF.
            ''' </summary>
            ''' <returns> Plain text document. </returns>
            Private Function ConvertToText() As String
                Dim pardNode As RtfTreeNode = Me.RootNode.FirstChild.SelectSingleChildNode("pard")

                Dim pPard As Integer = Me.RootNode.FirstChild.ChildNodes.IndexOf(pardNode)

                Dim enc As Encoding = Me.GetEncoding()

                Return ConvertToTextAux(Me.RootNode.FirstChild, pPard, enc)
            End Function

            ''' <summary>
            ''' Extract the text of a node RTF (Assistant ConvertToText ())
            ''' </summary>
            ''' <param name="curNode"> Current node. </param>
            ''' <param name="prim"> Node from which to convert. </param>
            ''' <param name="enc"> Encoding of the document. </param>
            ''' <returns> Plain text document. </returns>
            Private Function ConvertToTextAux(ByVal curNode As RtfTreeNode, ByVal prim As Integer, ByVal enc As Encoding) As String
                Dim res As New StringBuilder("")

                Dim nprin As RtfTreeNode = curNode

                Dim nodo As New RtfTreeNode()

                For i As Integer = prim To nprin.ChildNodes.Count - 1
                    nodo = nprin.ChildNodes(i)

                    If nodo.NodeType = RtfNodeType.Group Then
                        Dim indkw As Integer = If(nodo.FirstChild.NodeKey.Equals("*"), 1, 0)

                        If (Not nodo.ChildNodes(indkw).NodeKey.Equals("pict")) AndAlso (Not nodo.ChildNodes(indkw).NodeKey.Equals("object")) AndAlso (Not nodo.ChildNodes(indkw).NodeKey.Equals("fldinst")) Then
                            res.Append(ConvertToTextAux(nodo, 0, enc))
                        End If
                    ElseIf nodo.NodeType = RtfNodeType.Control Then
                        If nodo.NodeKey = "'" Then
                            res.Append(DecodeControlChar(nodo.Parameter, enc))
                        End If
                    ElseIf nodo.NodeType = RtfNodeType.Text Then
                        res.Append(nodo.NodeKey)
                    ElseIf nodo.NodeType = RtfNodeType.Keyword Then
                        If nodo.NodeKey.Equals("par") Then
                            res.AppendLine("")
                        End If
                    End If
                Next i

                Return res.ToString()
            End Function

            Private Function ParseStyleSheet(ByVal ssnode As RtfTreeNode) As RtfStyleSheet
                Dim rss As New RtfStyleSheet()

                For Each node As RtfTreeNode In ssnode.ChildNodes
                    If node.NodeKey = "cs" Then
                        rss.Type = RtfStyleSheetType.Character
                        rss.Index = node.Parameter
                    ElseIf node.NodeKey = "s" Then
                        rss.Type = RtfStyleSheetType.Paragraph
                        rss.Index = node.Parameter
                    ElseIf node.NodeKey = "ds" Then
                        rss.Type = RtfStyleSheetType.Section
                        rss.Index = node.Parameter
                    ElseIf node.NodeKey = "ts" Then
                        rss.Type = RtfStyleSheetType.Table
                        rss.Index = node.Parameter
                    ElseIf node.NodeKey = "additive" Then
                        rss.Additive = True
                    ElseIf node.NodeKey = "sbasedon" Then
                        rss.BasedOn = node.Parameter
                    ElseIf node.NodeKey = "snext" Then
                        rss.Next = node.Parameter
                    ElseIf node.NodeKey = "sautoupd" Then
                        rss.AutoUpdate = True
                    ElseIf node.NodeKey = "shidden" Then
                        rss.Hidden = True
                    ElseIf node.NodeKey = "slink" Then
                        rss.Link = node.Parameter
                    ElseIf node.NodeKey = "slocked" Then
                        rss.Locked = True
                    ElseIf node.NodeKey = "spersonal" Then
                        rss.Personal = True
                    ElseIf node.NodeKey = "scompose" Then
                        rss.Compose = True
                    ElseIf node.NodeKey = "sreply" Then
                        rss.Reply = True
                    ElseIf node.NodeKey = "styrsid" Then
                        rss.Styrsid = node.Parameter
                    ElseIf node.NodeKey = "ssemihidden" Then
                        rss.SemiHidden = True
                    ElseIf node.NodeType = RtfNodeType.Group AndAlso (node.ChildNodes(0).NodeKey = "*" AndAlso node.ChildNodes(1).NodeKey = "keycode") Then
                        rss.KeyCode = New RtfNodeCollection()

                        For i As Integer = 2 To node.ChildNodes.Count - 1
                            rss.KeyCode.Add(node.ChildNodes(i).CloneNode(True))
                        Next i
                    ElseIf node.NodeType = RtfNodeType.Text Then
                        rss.Name = node.NodeKey.Substring(0, node.NodeKey.Length - 1)
                    Else
                        If node.NodeKey <> "*" Then
                            rss.Formatting.Add(node)
                        End If
                    End If
                Next node

                Return rss
            End Function

#End Region

#Region "Properties"

            ''' <summary>
            ''' Returns the node of the tree of the document raz.
            ''' </summary>
            Public Property RootNode() As RtfTreeNode
                Get
                    'This returns the document node raz
                    Return m_RootNode
                End Get
                Set(ByVal value As RtfTreeNode)
                    m_RootNode = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the document's main group.
            ''' </summary>
            Public ReadOnly Property MainGroup() As RtfTreeNode
                Get
                    'This returns the main group (null if not available)
                    If m_RootNode.HasChildNodes() Then
                        Return m_RootNode.ChildNodes(0)
                    Else
                        Return Nothing
                    End If
                End Get
            End Property

            ''' <summary>
            ''' Returns the RTF text document.
            ''' </summary>
            Public ReadOnly Property Rtf() As String
                Get
                    'It returns the text of the full document RTF
                    Return m_RootNode.Rtf
                End Get
            End Property

            ''' <summary>
            ''' Indicates whether to decode the special characters (\ ') unindolos to contiguous text nodes.
            ''' </summary>
            Public Property MergeSpecialCharacters() As Boolean
                Get
                    Return  m_mergeSpecialCharacters
                End Get
                Set(ByVal value As Boolean)
                     m_mergeSpecialCharacters = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the plain text of the document.
            ''' </summary>
            Public ReadOnly Property Text() As String
                Get
                    Return ConvertToText()
                End Get
            End Property

#End Region
        End Class
    End Namespace
End Namespace
