'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: RtfTreeNode
'* Description: Node RTF representacin in the tree of a document.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.IO
Imports System.Text

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' RTF representacin node in the tree of a document.
        ''' </summary>
        Public Class RtfTreeNode

#Region "Private Attributes"

            ''' <summary>
            ''' Node Type.
            ''' </summary>
            Private type As RtfNodeType
            ''' <summary>
            ''' Keyword / Smbolo Control / Text.
            ''' </summary>
            Private key As String
            ''' <summary>
            ''' Indicates if the keyword has parmetro Control osmbolo.
            ''' </summary>
            Private hasParam As Boolean
            ''' <summary>
            ''' Parmetro of Control osmbolo keyword.
            ''' </summary>
            Private param As Integer
            ''' <summary>
            ''' Child nodes of current node.
            ''' </summary>
            Private children As RtfNodeCollection
            ''' <summary>
            ''' Parent node of current node.
            ''' </summary>
            Private parent As RtfTreeNode
            ''' <summary>
            ''' Raz node of the document.
            ''' </summary>
            Private root As RtfTreeNode
            ''' <summary>
            ''' Rtf tree node that owns the
            ''' </summary>
            Private m_tree As RtfTree

#End Region

#Region "Constructors Públicos"

            ''' <summary>
            ''' RtfTreeNode class constructor. Creates an uninitialized node.
            ''' </summary>
            Public Sub New()
                Me.type = RtfNodeType.None
                Me.key = ""

                'Initializing Default
                'this.param = 0;
                'this.hasParam = false;
                'this.parent = null;
                'this.root = null;
                'this.tree = null;
                'children = null;
            End Sub

            ''' <summary>
            ''' RtfTreeNode class constructor. Creates a node of a particular type.
            ''' </summary>
            ''' <param name="nodeType"> Type the node that is being created. </param>
            Public Sub New(ByVal nodeType As RtfNodeType)
                Me.type = nodeType
                Me.key = ""

                If nodeType = RtfNodeType.Group OrElse nodeType = RtfNodeType.Root Then
                    children = New RtfNodeCollection()
                End If

                If nodeType = RtfNodeType.Root Then
                    Me.root = Me
                End If

                'Initializing Default
                'this.param = 0;
                'this.hasParam = false;
                'this.parent = null;
                'this.root = null;
                'this.tree = null;
                'children = null;
            End Sub

            ''' <summary>
            ''' RtfTreeNode class constructor. Create a node specifying its type, keyword and parmetro.
            ''' </summary>
            ''' <param name="type"> Node Type. </param>
            ''' <param name="key"> Keyword osmbolo Control. </param>
            ''' <param name="hasParameter"> Indicates whether or keyword will smbolo Control was accompanied by a parmetro. </param>
            ''' <param name="parameter"> Parmetro of the keyword osmbolo Control. </param>
            Public Sub New(ByVal type As RtfNodeType, ByVal key As String, ByVal hasParameter As Boolean, ByVal parameter As Integer)
                Me.type = type
                Me.key = key
                Me.hasParam = hasParameter
                Me.param = parameter

                If type = RtfNodeType.Group OrElse type = RtfNodeType.Root Then
                    children = New RtfNodeCollection()
                End If

                If type = RtfNodeType.Root Then
                    Me.root = Me
                End If

                'Initializing Default
                'this.parent = null;
                'this.root = null;
                'this.tree = null;
                'children = null;
            End Sub

#End Region

#Region "Constructor Private"

            ''' <summary>
            ''' Private Constructor RtfTreeNode class. Create a node from a token lxico analyzer.
            ''' </summary>
            ''' <param name="token"> Token RTF lxico returned by the analyzer. </param>
            Friend Sub New(ByVal token As RtfToken)
                Me.type = CType(token.Type, RtfNodeType)
                Me.key = token.Key
                Me.hasParam = token.HasParameter
                Me.param = token.Parameter

                'Initializing Default
                'this.parent = null;
                'this.root = null;
                'this.tree = null;
                'this.children = null;
            End Sub

#End Region

#Region "alternative methods of Públicos"

            ''' <summary>
            ''' AADE a node at the end of the list of children.
            ''' </summary>
            ''' <param name="newNode"> New node AAAD. </param>
            Public Sub AppendChild(ByVal newNode As RtfTreeNode)
                If newNode IsNot Nothing Then
                    'If an tena no children is initialized Collection
                    If children Is Nothing Then
                        children = New RtfNodeCollection()
                    End If

                    'Parent node is assigned as the current node
                    newNode.parent = Me

                    'This update Root and Tree properties of the new node and its possible children
                    updateNodeRoot(newNode)

                    'It aade the new node at the end of the list of child nodes
                    children.Add(newNode)
                End If
            End Sub

            ''' <summary>
            ''' Inserts a new node in a given posicin the list of children.
            ''' </summary>
            ''' <param name="index"> Posicin in which to insert the node. </param>
            ''' <param name="newNode"> New node to insert. </param>
            Public Sub InsertChild(ByVal index As Integer, ByVal newNode As RtfTreeNode)
                If newNode IsNot Nothing Then
                    'If an tena no children is initialized Collection
                    If children Is Nothing Then
                        children = New RtfNodeCollection()
                    End If

                    If index >= 0 AndAlso index <= children.Count Then
                        'Parent node is assigned as the current node
                        newNode.parent = Me

                        'This update Root and Tree properties of the new node and its possible children
                        updateNodeRoot(newNode)

                        'It aade the new node at the end of the list of child nodes
                        children.Insert(index, newNode)
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Deletes a node from the list of children.
            ''' </summary>
            ''' <param name="index"> Index of node to delete. </param>
            Public Sub RemoveChild(ByVal index As Integer)
                'If the current node has children
                If children IsNot Nothing Then
                    If index >= 0 AndAlso index < children.Count Then
                        'It removes the nth child i
                        children.RemoveAt(index)
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Deletes a node from the list of children.
            ''' </summary>
            ''' <param name="node"> Node to delete. </param>
            Public Sub RemoveChild(ByVal node As RtfTreeNode)
                'If the current node has children
                If children IsNot Nothing Then
                    'It seeks to eliminate the node
                    Dim index As Integer = children.IndexOf(node)

                    'If we find
                    If index <> -1 Then
                        'It removes the nth child i
                        children.RemoveAt(index)
                    End If
                End If
            End Sub

            ''' <summary>
            ''' Make an exact copy of the current node.
            ''' </summary>
            ''' <param name="cloneChildren"> If this parmetro receives clonarn Also true is all the child nodes of current node. </param>
            ''' <returns> Returns an exact copy of the current node. </returns>
            Public Function CloneNode(ByVal cloneChildren As Boolean) As RtfTreeNode
                Dim clon As New RtfTreeNode()

                clon.key = Me.key
                clon.hasParam = Me.hasParam
                clon.param = Me.param
                clon.parent = Me.parent
                clon.root = Me.root
                clon.Tree = Me.m_tree
                clon.type = Me.type

                'If cloneChildren = false copied directly the list of children
                If (Not cloneChildren) Then
                    clon.children = Me.children
                Else 'Otherwise cloned Also each of the children, spreading the parmetro cloneChildren = true
                    clon.children = Nothing

                    If Me.children IsNot Nothing Then
                        clon.children = New RtfNodeCollection()

                        For Each child As RtfTreeNode In Me.children
                            clon.children.Add(child.CloneNode(True))
                        Next child
                    End If
                End If

                Return clon
            End Function

            ''' <summary>
            ''' Indicates whether the current node has child nodes.
            ''' </summary>
            ''' <returns> Returns true if the current node has child node algn. </returns>
            Public Function HasChildNodes() As Boolean
                Dim res As Boolean = False

                If children IsNot Nothing AndAlso children.Count > 0 Then
                    res = True
                End If

                Return res
            End Function

            ''' <summary>
            ''' Returns the first node of the list of child nodes of the current node whose key is specified as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> First node of the list of child nodes of the current node whose key is specified as parmetro. </returns>
            Public Function SelectSingleChildNode(ByVal keyword As String) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).key = keyword Then
                            node = children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the list of child nodes of the current node whose type is indicated as parmetro.
            ''' </summary>
            ''' <param name="nodeType"> Searched node type. </param>
            ''' <returns> First node of the list of child nodes of the current node whose type is indicated as parmetro. </returns>
            Public Function SelectSingleChildNode(ByVal nodeType As RtfNodeType) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).type = nodeType Then
                            node = children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the list of child nodes of the current node whose key and parmetro are those listed as parmetros.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <param name="param"> Parmetro searched. </param>
            ''' <returns> First node of the list of child nodes of the current node whose key and parmetro are those listed as parmetros. </returns>
            Public Function SelectSingleChildNode(ByVal keyword As String, ByVal param As Integer) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).key = keyword AndAlso children(i).param = param Then
                            node = children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node group from the list of child nodes of the current node whose first word is indicated as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> First group node from the list of child nodes of the current node whose first word is indicated as parmetro. </returns>
            Public Function SelectSingleChildGroup(ByVal keyword As String) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).NodeType = RtfNodeType.Group AndAlso children(i).HasChildNodes() AndAlso children(i).FirstChild.NodeKey = keyword Then
                            node = children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the tree, from the current node, whose type is indicated as parmetro.
            ''' </summary>
            ''' <param name="nodeType"> Searched node type. </param>
            ''' <returns> First node in the tree, from the current node, whose type is indicated as parmetro. </returns>
            Public Function SelectSingleNode(ByVal nodeType As RtfNodeType) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).type = nodeType Then
                            node = children(i)
                            found = True
                        Else
                            node = children(i).SelectSingleNode(nodeType)

                            If node IsNot Nothing Then
                                found = True
                            End If
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the tree, from the current node, whose key is specified as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> First node in the tree, from the current node, whose key is specified as parmetro. </returns>
            Public Function SelectSingleNode(ByVal keyword As String) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).key = keyword Then
                            node = children(i)
                            found = True
                        Else
                            node = children(i).SelectSingleNode(keyword)

                            If node IsNot Nothing Then
                                found = True
                            End If
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the tree group, from the current node, the first keyword is indicated as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> First node of the tree group, from the current node, the first keyword is indicated as parmetro. </returns>
            Public Function SelectSingleGroup(ByVal keyword As String) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).NodeType = RtfNodeType.Group AndAlso children(i).HasChildNodes() AndAlso children(i).FirstChild.NodeKey = keyword Then
                            node = children(i)
                            found = True
                        Else
                            node = children(i).SelectSingleGroup(keyword)

                            If node IsNot Nothing Then
                                found = True
                            End If
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the first node of the tree, from the current node, whose key and parmetro are those listed as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <param name="param"> Parmetro searched. </param>
            ''' <returns> First node in the tree, from the current node, whose key and parmetro are indicated as parmetro ls. </returns>
            Public Function SelectSingleNode(ByVal keyword As String, ByVal param As Integer) As RtfTreeNode
                Dim i As Integer = 0
                Dim found As Boolean = False
                Dim node As RtfTreeNode = Nothing

                If children IsNot Nothing Then
                    Do While i < children.Count AndAlso Not found
                        If children(i).key = keyword AndAlso children(i).param = param Then
                            node = children(i)
                            found = True
                        Else
                            node = children(i).SelectSingleNode(keyword, param)

                            If node IsNot Nothing Then
                                found = True
                            End If
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns all nodes from current node, whose key is specified as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> Collection of nodes from current node, whose key is specified as parmetro. </returns>
            Public Function SelectNodes(ByVal keyword As String) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.key = keyword Then
                            nodes.Add(node)
                        End If

                        nodes.AddRange(node.SelectNodes(keyword))
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes group, from the current node, the first keyword is indicated as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> Collection of nodes group, from the current node, the first keyword is indicated as parmetro. </returns>
            Public Function SelectGroups(ByVal keyword As String) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.NodeType = RtfNodeType.Group AndAlso node.HasChildNodes() AndAlso node.FirstChild.NodeKey = keyword Then
                            nodes.Add(node)
                        End If

                        nodes.AddRange(node.SelectGroups(keyword))
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes from current node, whose type is indicated as parmetro.
            ''' </summary>
            ''' <param name="nodeType"> Searched node type. </param>
            ''' <returns> Collection of nodes from current node, whose type is indicated as parmetro. </returns>
            Public Function SelectNodes(ByVal nodeType As RtfNodeType) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.type = nodeType Then
                            nodes.Add(node)
                        End If

                        nodes.AddRange(node.SelectNodes(nodeType))
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes from current node, whose key and parmetro are those listed as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <param name="param"> Parmetro searched. </param>
            ''' <returns> Collection of nodes from current node, whose key and parmetro are those listed as parmetro. </returns>
            Public Function SelectNodes(ByVal keyword As String, ByVal param As Integer) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.key = keyword AndAlso node.param = param Then
                            nodes.Add(node)
                        End If

                        nodes.AddRange(node.SelectNodes(keyword, param))
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes in the list of child nodes of the current node whose key is specified as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> Collection of nodes in the list of child nodes of the current node whose key is specified as parmetro. </returns>
            Public Function SelectChildNodes(ByVal keyword As String) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.key = keyword Then
                            nodes.Add(node)
                        End If
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all node groups in the list of child nodes of the current node whose first word is indicated as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> Collection of nodes group from the list of child nodes of the current node whose first word is indicated as parmetro. </returns>
            Public Function SelectChildGroups(ByVal keyword As String) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.NodeType = RtfNodeType.Group AndAlso node.HasChildNodes() AndAlso node.FirstChild.NodeKey = keyword Then
                            nodes.Add(node)
                        End If
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes in the list of child nodes of the current node whose type is indicated as parmetro.
            ''' </summary>
            ''' <param name="nodeType"> Searched node type. </param>
            ''' <returns> Collection of nodes in the list of child nodes of the current node whose type is indicated as parmetro. </returns>
            Public Function SelectChildNodes(ByVal nodeType As RtfNodeType) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.type = nodeType Then
                            nodes.Add(node)
                        End If
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns all nodes in the list of child nodes of the current node whose key and parmetro are those listed as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <param name="param"> Parmetro searched. </param>
            ''' <returns> Collection of nodes in the list of child nodes of the current node whose key and parmetro are those listed as parmetro. </returns>
            Public Function SelectChildNodes(ByVal keyword As String, ByVal param As Integer) As RtfNodeCollection
                Dim nodes As New RtfNodeCollection()

                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.key = keyword AndAlso node.param = param Then
                            nodes.Add(node)
                        End If
                    Next node
                End If

                Return nodes
            End Function

            ''' <summary>
            ''' Returns the next sibling of the current node whose key is specified as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> Brother of the current first node whose key is specified as parmetro. </returns>
            Public Function SelectSibling(ByVal keyword As String) As RtfTreeNode
                Dim node As RtfTreeNode = Nothing
                Dim par As RtfTreeNode = Me.parent

                If par IsNot Nothing Then
                    Dim curInd As Integer = par.ChildNodes.IndexOf(Me)

                    Dim i As Integer = curInd + 1
                    Dim found As Boolean = False

                    Do While i < par.children.Count AndAlso Not found
                        If par.children(i).key = keyword Then
                            node = par.children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the next sibling of the current node whose type is indicated as parmetro.
            ''' </summary>
            ''' <param name="nodeType"> Tpo searched node. </param>
            ''' <returns> Brother of the current first node whose type is indicated as parmetro. </returns>
            Public Function SelectSibling(ByVal nodeType As RtfNodeType) As RtfTreeNode
                Dim node As RtfTreeNode = Nothing
                Dim par As RtfTreeNode = Me.parent

                If par IsNot Nothing Then
                    Dim curInd As Integer = par.ChildNodes.IndexOf(Me)

                    Dim i As Integer = curInd + 1
                    Dim found As Boolean = False

                    Do While i < par.children.Count AndAlso Not found
                        If par.children(i).type = nodeType Then
                            node = par.children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' Returns the next sibling node of the current keyword and parmetro which are those listed as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <param name="param"> Parmetro searched. </param>
            ''' <returns> Brother of the current first node whose key and parmetro are those listed as parmetro. </returns>
            Public Function SelectSibling(ByVal keyword As String, ByVal param As Integer) As RtfTreeNode
                Dim node As RtfTreeNode = Nothing
                Dim par As RtfTreeNode = Me.parent

                If par IsNot Nothing Then
                    Dim curInd As Integer = par.ChildNodes.IndexOf(Me)

                    Dim i As Integer = curInd + 1
                    Dim found As Boolean = False

                    Do While i < par.children.Count AndAlso Not found
                        If par.children(i).key = keyword AndAlso par.children(i).param = param Then
                            node = par.children(i)
                            found = True
                        End If

                        i += 1
                    Loop
                End If

                Return node
            End Function

            ''' <summary>
            ''' For all nodes of type text containing the search text.
            ''' </summary>
            ''' <param name="text"> Search in the document. </param>
            ''' <returns> Node list, as of today, which contain the search text. </returns>
            Public Function FindText(ByVal text As String) As RtfNodeCollection
                Dim list As New RtfNodeCollection()

                'If the current node has children
                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.NodeType = RtfNodeType.Text AndAlso node.NodeKey.IndexOf(text) <> -1 Then
                            list.Add(node)
                        ElseIf node.NodeType = RtfNodeType.Group Then
                            list.AddRange(node.FindText(text))
                        End If
                    Next node
                End If

                Return list
            End Function

            ''' <summary>
            ''' Search and replace specific text in all nodes of type from the existing text.
            ''' </summary>
            ''' <param name="oldValue"> Search. </param>
            ''' <param name="newValue"> Replacement text. </param>
            Public Sub ReplaceText(ByVal oldValue As String, ByVal newValue As String)
                'If the current node has children
                If children IsNot Nothing Then
                    For Each node As RtfTreeNode In children
                        If node.NodeType = RtfNodeType.Text Then
                            node.NodeKey = node.NodeKey.Replace(oldValue, newValue)
                        ElseIf node.NodeType = RtfNodeType.Group Then
                            node.ReplaceText(oldValue, newValue)
                        End If
                    Next node
                End If
            End Sub

            ''' <summary>
            ''' Returns a node representacin which indicates its type, key indicator of value parmetro and parmetro
            ''' </summary>
            ''' <returns> Character string type [TYPE, KEY, IND_PARAMETRO, VAL_PARAMETRO] </returns>
            Public Overrides Function ToString() As String
                Return "[" & Me.type.ToString() & ", " & Me.key & ", " & Me.hasParam & ", " & Me.param & "]"
            End Function

#End Region

#Region "Private Methods"

            ''' <summary>
            ''' Gets the text from the RTF representacin current node in the tree.
            ''' </summary>
            ''' <returns> RTF text node. </returns>
            Private Function getRtf() As String
                Dim res As String = ""

                Dim enc As Encoding = Me.m_tree.GetEncoding()

                res = getRtfInm(Me, Nothing, enc)

                Return res
            End Function

            ''' <summary>
            ''' Alternative methods of aid to get the RTF text from the current node in your tree representacin.
            ''' </summary>
            ''' <param name="curNode"> Current node tree. </param>
            ''' <param name="prevNode"> Previous node treaty. </param>
            ''' <param name="enc"> Encoding of the document. </param>
            ''' <returns> RTF text node. </returns>
            Private Function getRtfInm(ByVal curNode As RtfTreeNode, ByVal prevNode As RtfTreeNode, ByVal enc As Encoding) As String
                Dim res As New StringBuilder("")

                If curNode.NodeType = RtfNodeType.Root Then
                    res.Append("")
                ElseIf curNode.NodeType = RtfNodeType.Group Then
                    res.Append("{")
                Else
                    If curNode.NodeType = RtfNodeType.Control OrElse curNode.NodeType = RtfNodeType.Keyword Then
                        res.Append("\")
                    Else 'curNode.NodeType == RtfNodeType.Text
                        If prevNode IsNot Nothing AndAlso prevNode.NodeType = RtfNodeType.Keyword Then
                            Dim code As Integer = Char.ConvertToUtf32(curNode.NodeKey, 0)

                            If code >= 32 AndAlso code < 128 Then
                                res.Append(" ")
                            End If
                        End If
                    End If

                    AppendEncoded(res, curNode.NodeKey, enc)

                    If curNode.HasParameter Then
                        If curNode.NodeType = RtfNodeType.Keyword Then
                            res.Append(Convert.ToString(curNode.Parameter))
                        ElseIf curNode.NodeType = RtfNodeType.Control Then
                            'If it is a special character as the accented vowels
                            If curNode.NodeKey = "'" Then
                                res.Append(GetHexa(curNode.Parameter))
                            End If
                        End If
                    End If
                End If

                'You get the child nodes
                Dim children As RtfNodeCollection = curNode.ChildNodes

                'If the node has children obtained the code of the children RTF
                If children IsNot Nothing Then
                    For i As Integer = 0 To children.Count - 1
                        Dim node As RtfTreeNode = children(i)

                        If i > 0 Then
                            res.Append(getRtfInm(node, children(i - 1), enc))
                        Else
                            res.Append(getRtfInm(node, Nothing, enc))
                        End If
                    Next i
                End If

                If curNode.NodeType = RtfNodeType.Group Then
                    res.Append("}")
                End If

                Return res.ToString()
            End Function

            ''' <summary>
            ''' Concatenate two strings using the Encoding of the document.
            ''' </summary>
            ''' <param name="res"> Original string. </param>
            ''' <param name="s"> Chain AAAD. </param>
            ''' <param name="enc"> Encoding of the document. </param>
            Private Sub AppendEncoded(ByVal res As StringBuilder, ByVal s As String, ByVal enc As Encoding)
                'Contributed by Jan Stuchlk

                For i As Integer = 0 To s.Length - 1
                    Dim code As Integer = Char.ConvertToUtf32(s, i)

                    If code >= 128 OrElse code < 32 Then
                        res.Append("\'")
                        Dim bytes() As Byte = enc.GetBytes(New Char() {s.Chars(i)})
                        res.Append(GetHexa(bytes(0)))
                    Else
                        If (s.Chars(i) = "{"c) OrElse (s.Chars(i) = "}"c) OrElse (s.Chars(i) = "\"c) Then
                            res.Append("\")
                        End If

                        res.Append(s.Chars(i))
                    End If
                Next i
            End Sub

            ''' <summary>
            ''' Gets the code of a hexadecimal integer.
            ''' </summary>
            ''' <param name="code"> NUMBER integer. </param>
            ''' <returns> Cdigo hexadecimal integer passed as parmetro. </returns>
            Private Function GetHexa(ByVal code As Integer) As String
                'Contributed by Jan Stuchlk

                Dim hexa As String = Convert.ToString(code, 16)

                If hexa.Length = 1 Then
                    hexa = "0" & hexa
                End If

                Return hexa
            End Function

            ''' <summary>
            ''' Update Root and Tree properties of a node (and children) with the current node.
            ''' </summary>
            ''' <param name="node"> Node to update. </param>
            Private Sub updateNodeRoot(ByVal node As RtfTreeNode)
                'Raz node is assigned to document
                node.root = Me.root

                'Owner is assigned the node tree
                node.Tree = Me.m_tree

                'If the current node has children Also updated
                If node.children IsNot Nothing Then
                    'They are updated recursively the children of current node
                    For Each nod As RtfTreeNode In node.children
                        updateNodeRoot(nod)
                    Next nod
                End If
            End Sub

#End Region

#Region "Properties"

            ''' <summary>
            ''' Returns the node of the tree of the document raz.
            ''' </summary>
            ''' <remarks>
            ''' T his is not the raz the tree node, but it's just a dummy node ROOT rate of departure of the rest of the tree RTF.
            ''' Consequently have a single child node of type GROUP, real root of the tree.
            ''' </remarks>
            Public Property RootNode() As RtfTreeNode
                Get
                    Return root
                End Get
                Set(ByVal value As RtfTreeNode)
                    root = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the parent node of current node.
            ''' </summary>
            Public Property ParentNode() As RtfTreeNode
                Get
                    Return parent
                End Get
                Set(ByVal value As RtfTreeNode)
                    parent = value
                End Set
            End Property

            ''' <summary>
            ''' Rtf Returns the tree node to which it belongs.
            ''' </summary>
            Public Property Tree() As RtfTree
                Get
                    Return m_tree
                End Get
                Set(ByVal value As RtfTree)
                    m_tree = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the type of the current node.
            ''' </summary>
            Public Property NodeType() As RtfNodeType
                Get
                    Return type
                End Get
                Set(ByVal value As RtfNodeType)
                    type = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the keyword, or Text Control smbolo the current node.
            ''' </summary>
            Public Property NodeKey() As String
                Get
                    Return key
                End Get
                Set(ByVal value As String)
                    key = value
                End Set
            End Property

            ''' <summary>
            ''' Indicates whether the current node has assigned parmetro.
            ''' </summary>
            Public Property HasParameter() As Boolean
                Get
                    Return hasParam
                End Get
                Set(ByVal value As Boolean)
                    hasParam = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the parmetro assigned to the current node.
            ''' </summary>
            Public Property Parameter() As Integer
                Get
                    Return param
                End Get
                Set(ByVal value As Integer)
                    param = value
                End Set
            End Property

            ''' <summary>
            ''' Returns the collection of child nodes of current node.
            ''' </summary>
            Public Property ChildNodes() As RtfNodeCollection
                Get
                    Return children
                End Get
                Set(ByVal value As RtfNodeCollection)
                    children = value

                    For Each node As RtfTreeNode In children
                        node.parent = Me

                        'This update Root and Tree properties of the new node and its possible children
                        updateNodeRoot(node)
                    Next node
                End Set
            End Property

            ''' <summary>
            ''' Returns the first child node whose key is indicated as parmetro.
            ''' </summary>
            ''' <param name="keyword"> Searched keyword. </param>
            ''' <returns> First child node whose key is indicated as parmetro. In the absence of null is returned. </returns>
            Default Public ReadOnly Property Item(ByVal keyword As String) As RtfTreeNode
                Get
                    Return Me.SelectSingleChildNode(keyword)
                End Get
            End Property

            ''' <summary>
            ''' Returns the nth child of the current node.
            ''' </summary>
            ''' <param name="childIndex"> index of child node to retrieve. </param>
            ''' <returns> Nth child node of current node. Returns null if not exist. </returns>
            Default Public ReadOnly Property Item(ByVal childIndex As Integer) As RtfTreeNode
                Get
                    Dim res As RtfTreeNode = Nothing

                    If children IsNot Nothing AndAlso childIndex >= 0 AndAlso childIndex < children.Count Then
                        res = children(childIndex)
                    End If

                    Return res
                End Get
            End Property

            ''' <summary>
            ''' Returns the first child node of current node.
            ''' </summary>
            Public ReadOnly Property FirstChild() As RtfTreeNode
                Get
                    Dim res As RtfTreeNode = Nothing

                    If children IsNot Nothing AndAlso children.Count > 0 Then
                        res = children(0)
                    End If

                    Return res
                End Get
            End Property

            ''' <summary>
            ''' Ltimo Returns the child node of current node.
            ''' </summary>
            Public ReadOnly Property LastChild() As RtfTreeNode
                Get
                    Dim res As RtfTreeNode = Nothing

                    If children IsNot Nothing AndAlso children.Count > 0 Then
                        Return children(children.Count - 1)
                    End If

                    Return res
                End Get
            End Property

            ''' <summary>
            ''' Returns the next sibling node of the current node (Two nodes are siblings if they have the same parent node [ParentNode]).
            ''' </summary>
            Public ReadOnly Property NextSibling() As RtfTreeNode
                Get
                    Dim res As RtfTreeNode = Nothing

                    If parent IsNot Nothing AndAlso parent.children IsNot Nothing Then
                        Dim currentIndex As Integer = parent.children.IndexOf(Me)

                        If parent.children.Count > currentIndex + 1 Then
                            res = parent.children(currentIndex + 1)
                        End If
                    End If

                    Return res
                End Get
            End Property

            ''' <summary>
            ''' Returns the previous sibling node of the current node (Two nodes are siblings if they have the same parent node [ParentNode]).
            ''' </summary>
            Public ReadOnly Property PreviousSibling() As RtfTreeNode
                Get
                    Dim res As RtfTreeNode = Nothing

                    If parent IsNot Nothing AndAlso parent.children IsNot Nothing Then
                        Dim currentIndex As Integer = parent.children.IndexOf(Me)

                        If currentIndex > 0 Then
                            res = parent.children(currentIndex - 1)
                        End If
                    End If

                    Return res
                End Get
            End Property

            ''' <summary>
            ''' RTF Returns the code of the current node and all its child nodes.
            ''' </summary>
            Public ReadOnly Property Rtf() As String
                Get
                    Return getRtf()
                End Get
            End Property

            ''' <summary>
            ''' Returns the index of the current node in the list of children of its parent node.
            ''' </summary>
            Public ReadOnly Property Index() As Integer
                Get
                    Dim res As Integer = -1

                    If parent IsNot Nothing Then
                        res = parent.children.IndexOf(Me)
                    End If

                    Return res
                End Get
            End Property

#End Region

        End Class
    End Namespace
End Namespace
