'************************************************* ******************************
'* This file is part of NRtfTree Library.
'*
'* NRtfTree Library is free software; you can redistribute it and / or modify
'* It under the terms of the GNU Lesser General Public License as published by
'* The Free Software Foundation, either version 3 of the License, or
'* (At your option) any later version.
'*
'* NRtfTree Library is distributed in the hope that it will be useful,
'* But WITHOUT ANY WARRANTY; without even the implied warranty of
'* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'* GNU Lesser General Public License for more details.
'*
'* You should have received a copy of the GNU Lesser General Public License
'* Along with this program. If not, see <http://www.gnu.org/licenses/> .
'************************************************* ******************************

'************************************************* ******************************
'* Library: NRtfTree
'* Version: v0.3.0
'* Date: 01/05/2009
'* Copyright: 2006-2009 Salvador Gomez
'* E-mail: sgoliver.net @ gmail.com
'* Home Page: http://www.sgoliver.net
'* SF Project: http://nrtftree.sourceforge.net
'* Http://sourceforge.net/projects/nrtftree
'* Class: SarParser
'* Description: Processor abstract class used by RtfReader.
'* VB.NET Version Contributed by: Terence L. Wallace
'* ************************************************ *****************************


Imports Microsoft.VisualBasic
Imports System

Namespace Net.Sgoliver.NRtfTree
    Namespace Core
        ''' <summary>
        ''' This class, used by RtfReader, contains all the methods, described to deal with each one of
        ''' types of elements in an RTF document. These methods, described SERN called automticamente
        ''' during the RTF document anlisis done by RtfReader class.
        ''' </summary>
        Public MustInherit Class SarParser
            ''' <summary>
            ''' This is called an alternative methods of sla anlisis once at the beginning of the RTF document.
            ''' </summary>
            Public MustOverride Sub StartRtfDocument()
            ''' <summary>
            ''' This alternative methods of sla is called a session at the end anlisis RTF document.
            ''' </summary>
            Public MustOverride Sub EndRtfDocument()
            ''' <summary>
            ''' The methods, described is called whenever a key is read from beginning RTF group.
            ''' </summary>
            Public MustOverride Sub StartRtfGroup()
            ''' <summary>
            ''' The methods, described is called each time you read a key group to RTF.
            ''' </summary>
            Public MustOverride Sub EndRtfGroup()
            ''' <summary>
            ''' The methods, described is called whenever a keyword is read RTF.
            ''' </summary>
            ''' <param name="key"> Keyword read the document. </param>
            ''' <param name="hasParameter"> Indicates whether the keyword is a parmetro acompaada. </param>
            ''' <param name="parameter">
            ''' Parmetro that accompany the keyword. If the keyword is not going acompaada
            ''' of ningn parmetro, ie that the field is hasParam 'false',
            ''' this field contains the value 0.
            ''' </param>
            Public MustOverride Sub RtfKeyword(ByVal key As String, ByVal hasParameter As Boolean, ByVal parameter As Integer)
            ''' <summary>
            ''' The methods, described is called each time you read an RTF Control smbolo.
            ''' </summary>
            ''' <param name="key"> Control Smbolo read the document. </param>
            ''' <param name="hasParameter"> Indicates whether the Control is smbolo was accompanied by a parmetro. </param>
            ''' <param name="parameter">
            ''' Parmetro that accompanies the smbolo Control. If that does not go smbolo acompaada Control
            ''' of ningn parmetro, ie that the field is hasParam 'false',
            ''' this field contains the value 0.
            ''' </param>
            Public MustOverride Sub RtfControl(ByVal key As String, ByVal hasParameter As Boolean, ByVal parameter As Integer)
            ''' <summary>
            ''' The methods, described is called each time you read a snippet of text from an RTF document.
            ''' </summary>
            ''' <param name="text"> Text read from the document. </param>
            Public MustOverride Sub RtfText(ByVal text As String)
        End Class
    End Namespace
End Namespace
